'960412 1437
'****************************************************************************
'* PROGRAM NAME: CCDCONVT.BAS   DATE: 13. 9.1992   INCLUDE IN: is main      *
'* LANGUAGE: Power BASIC 3.20  (Robert S.Zale)                              *
'* PURPOSE: Converts between different CCD picture formats.                 *
'* INCLUDE FILES: ERROR3  .INC  Fatal error handler                         *
'*                IOTEXT  .INC  Set of Text and File functions              *
'*                CCDCPROC.INC  Procedures                                  *
'*                CCDCFUNC.INC  Functions                                   *
'*                CCDCDAT .INC  English and German Texts                    *
'*                CCDUSUB .INC  Input/Output subroutines                    *
'* OBJECT FILES:  PBSTACK. OBJ  Power BASIC 2 & 3 Stack handler for objects *
'*                ARRAYEXT.PBL  Advanced array manipulation functions)      *
'*                CLRSCRN .PBL  Collection of clear screen routines         *
'*                CCDCC   .OBJ  procedures for CCDUTIL and CCDCONVT         *
'*                RLCEGEN .OBJ  Run Length Count Encoding Generator         *
'* UNIT FILES:    PB3GPUSE.PBU  General Purpose Functions and Procedures    *
'* INPUT FILES:   filename.LNX  Lynxx image                                 *
'*                filename.ALI  Artificial Lynxx Image                      *
'*                filename.ST4  ST-4 image                                  *
'*                filename.ST6  ST-6 image                                  *
'*                filename.ASI  Artificial ST-4 Image                       *
'*                filename.BYT  CCDUTIL image format                        *
'*                filename.PIX  QuickPix format                             *
'*                filename.FIT  FITS format                                 *
'* OUTPUT FILES:  filename.LNX  Lynxx image                                 *
'*                filename.ALI  Artificial Lynxx Image                      *
'*                filename.ST4  ST-4 image                                  *
'*                filename.ST6  ST-6 image                                  *
'*                filename.ASI  Artificial ST-4 Image                       *
'*                filename.BYT  CCDUTIL image format                        *
'*                filename.PIX  QuickPix format                             *
'*                filename.FIT  FITS format                                 *
'*                filename.nnC  any, copy of original                       *
'*                filename.nn1  any, rotated  90 CCW = 270 CW             *
'*                filename.nn2  any, rotated 180 CCW = 180 CW             *
'*                filename.nn3  any, rotated 270 CCW =  90 CW             *
'*                filename.nnU  any, Y-axis flipped: Upside down            *
'*                filename.nnM  any, X-axis flipped: Mirrored               *
'*                filename.nnF  any, X-/Y-axis flipped: Flipped             *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.74   8-APR-96: Clear Screen routines in CLRSCRN.PBL added     *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.73  10-FEB-96: Made it FreeWare                               *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.72   2-SEP-95: Recompiled from PB3.10 to PB3.20               *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.71  17-FEB-95: Must now be registered                         *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.63  25-JUN-94: Added FITS format                              *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.53  1-JUN-94: RLCE-Encoder overwholed.                        *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.43 18-JAN-94: Added ST6 conversion. All input files have now  *
'*          all processing and output options. Input files are all arranged *
'*          column-row wise with 12 bit resolution. Output file option      *
'*          cares for bit reduction/expansion and row/column rearrangement. *
'*          A couple of minor bugs have been fixed on the fly as well.      *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.33 31-DEC-93: BYT files are always RLCE encoded               *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.23 28-DEC-93: Added RLCE file compression/decompression       *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.13  3-SEP-93: Added QuickPix format PIX (Richard Berry)       *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.03 21-MAY-93: Competely overwholed to fit Version 2 series.   *
'*               Last digit indicates now PB Version, here 3.               *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.07 22-FEB-93: Adjusted language selection to match batch and  *
'*          made it pushbuttonoid.                                          *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.06  2-JAN-93: Processing option "0" had a bug.                *
'*          Also, the Lynxx synthesizer had a minor but "lethal" bug.       *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.05 14-NOV-92: Added justify option when rotating and fixed a  *
'*          a very nasty bug in C12T16 and SynthLynx procedures.            *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.04 18-OCT-92: Bilingual                                       *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.03 17-OCT-92: Permits all image types to be flipped.          *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.02 21-SEP-92: It needs a new version. The lynx synthesizer    *
'*          has been changed to make the synthesized image look like the    *
'*          original when viewed with Lynxx program in VGA mode.            *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.01 20-SEP-92: Output path query after input path query. All   *
'*          texts now in the data area.                                     *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.00 13-SEP-92: First working version.                          *
'* ------------------------------------------------------------------------ *
'* COMMENTS: If LNX is to be converted to ALI or vice versa, or ST4 to ASI, *
'*           or vice versa, the file is only copied, using DOS COPY. If     *
'*           input and output options do match, nothing will be done.       *
'* ------------------------------------------------------------------------ *
'* NOTE: For file formats, see listing of CCDUTIL.                          *
'*       SET ENGLISH% ACCORDING TO LANGUAGE                                 *
'* ======================================================================== *
'* AUTHOR: Hans-Ruedi H. Wernli, Pletschgasse, CH-3952 Susten, Switzerland. *
'****************************************************************************
'
'
' COMPILER DIRECTIVES
'----------------------------------------------------------------------------
'
'$alias data as "DSEG"	'or data segment name as used in .OBJ
'$code seg "CODE"	'or code segment name as used in .OBJ
'$huge			'for arrays > 64K
$cpu 8086		'8086/80286/80386
$debug map off		'on/off
$debug path off		'on/off
$debug unit off		'on/off
$debug pbdebug off	'on/off
$dim array		'all/array/none
$dynamic		'$static
$error all off		'bounds/numeric/overflow/stack/all on/off
$event off		'on/off
$float procedure	'emulate/npx/procedure
$lib com-,lpt-,graph-,cga-,ega-,vga-,herc-,fullfloat-,iprint-
$optimize speed		'size/speed
$option cntlbreak off	'cntlbreak/gosub on/off
$stack 4096		'1536...32767
$com 0			'0...32767
$sound 1		'1...4096
$string 4		'1/2/4/8/16/32
$compile exe		'memory/exe/chain/unit
'
'
' LANGUAGE SELECTION AND VERSION NUMBER
'----------------------------------------------------------------------------
'
VN$        = "CCDCONVT V.2.78"
CCDConvtV% = 278
on error goto ErrorHandling
%True   = -1
%False  =  0
Chroma% = %True
shared Idiom%()
dim Idiom%(0:2)			'0:E=true/D=false 1:SL-text-, 2:ML text-offs
English% = ComLinLan%(command$,"CCDTOOLS: "+VN$)
if English% then
  Idiom%(0) = %True		'English
  Idiom%(1) = 0			'SL offset
  Idiom%(2) = 0			'ML offset
  Version$ = "CCD Image File Format Converter " + VN$
else
  Idiom%(0) = %False		'Deutsch
  Idiom%(1) = 100		'SL offset
  Idiom%(2) = 20		'ML offset
  Version$ = "CCD Bild Datei Format Konverter " + VN$
end if
'
'
' CONSTANTS COLORS
'-----------------------------------------------------------------------------
'
%Black     =  0        'COLORS, dark
%Blue      =  1
%Green     =  2
%Cyan      =  3
%Red       =  4
%Pink      =  5
%Brown     =  6
%White     =  7
%Gray      =  8        'COLORS, light
%LBlue     =  9
%LGreen    = 10
%LCyan     = 11
%LRed      = 12
%LPink     = 13
%Yellow    = 14
%LWhite    = 15
'
%DOSRead  = 63		'DOS function used at Encode/Decode for ST4 and ASI
%DOSWrite = 64		'  EDCST4
'
'
' OTHER CONSTANTS AND VARIABLES DECLARATIONS
'----------------------------------------------------------------------------
'
%LMN       = 31680     'eLeMeNts in arrays
%UnitAUX   = 1         'Only one file is open at a time, except for FITS
%UnitFIT   = 2	       'FITS header, if required
'
Chroma%  = %True	'TRUE if colors available (for ERROR.INC)
on error goto ErrorHandling
'
'
' Function and Procedure Declaration in external object: ARRAYEXT.ASM/OBJ
'----------------------------------------------------------------------------
'
declare function ArrayExtV%()
declare sub ShlWrd(integer,integer)
declare sub ShrWrd(integer,integer)
declare sub XDimBA(integer,integer,integer,integer)
declare sub XDimWA(integer,integer,integer,integer)
declare sub CPYW2B(integer,integer,integer)
declare sub CPYB2W(integer,integer,integer)
declare sub CPYW2W(integer,integer,integer)
declare sub FillWC(integer,integer,integer)
declare sub LmtHWC(integer,integer,integer)
declare sub SubCWL(integer,word,integer,integer)
declare sub ShlMWA(integer,integer,integer)
declare sub ShrMWA(integer,integer,integer)
declare sub StrVar(integer,integer,integer,string)
declare sub MotInt(integer,integer)
declare function MINWRD%(integer,integer)
declare function MAXWRD%(integer,integer)
'
'
' Function and Procedure Declaration in external units: CLRSCRN.PBL
'----------------------------------------------------------------------------
'
declare function ClrScrnV%()
declare sub ClsCE()
declare sub ClsTB()
declare sub HoroSoftLogo()
'
'
' Function and Procedure Declaration in external units: PB3GPUSE.INC/PBU
'----------------------------------------------------------------------------
'
declare function PB3GPUseV%()
declare function PBVer%(string)
declare function EscRet%()
declare function AddSlash$(string)
declare function Center$(string,integer)
declare function LogOn$(string,integer,integer)
declare function DateNorm$(integer)
declare function KeyIn$()
declare function ComLinLan%(string,string)
declare function IOPath$(string,string,string)
declare function IOFile$(integer,integer,string)
declare function Confirm%(string)
declare function Redo%(string)
declare function FileHere%(string)
declare function SizeOk%(string,long)
declare function ByteBin$(integer)
declare function Outch$(string)
declare sub ClrLine()
declare sub Illegal(string,string)
'
'
' Function and Procedure Declaration in external units: CCDCC.ASM/OBJ
'----------------------------------------------------------------------------
'
declare function CCDCCV%()
declare sub DecLnx(integer,integer)
declare sub EncLnx(integer,integer)
declare sub EDCST4(integer,string,integer,integer)
declare sub EDCPix(integer,integer,integer)
declare sub FLIPXY(integer,integer)
declare sub RoTwic(integer,integer)
declare sub MovLft(integer,integer)
declare sub TwoGet(integer,integer,word,word)
declare sub TwoBin(integer,integer,word)
declare sub AbsIAH(integer,integer)
declare function TwoLU%(integer,integer)
declare function TwoLC%(integer,integer,integer)
declare function HotSpt%(integer,word)
declare function Delta%(integer,integer,integer,integer)
declare function RdFit%(integer,integer,integer,integer)
declare function WrFits%(integer,integer,integer)
'
'
' Function and Procedure Declaration in external units: RLCEGEN.ASM/OBJ
'----------------------------------------------------------------------------
'
declare function RlceGenV%()
declare function EncBie%(integer,integer,word)
declare function WrBie%(integer,integer)
declare function DecBie%(integer,integer)
declare function RdBie%(integer,integer,word)
'
'
' FILES TO BE LINKED IN
'-----------------------------------------------------------------------------
'
$link "PBSTACK.OBJ"
$link "ARRAYEXT.PBL"
$link "CLRSCRN.PBL"
$link "CCDCC.OBJ"
$link "RLCEGEN.OBJ"
$link "PB3GPUSE.PBU"
'
'
' FILES TO BE INCLUDED
'-----------------------------------------------------------------------------
'
$include "CCDCPROC.INC"
$include "CCDCFUNC.INC"
$include "IOTEXT.INC"
$include "CCDCSUB.INC"
$include "CCDCDAT.INC"
$include "ERROR3.INC"
'
'
' VARIABLES SETUP
'-----------------------------------------------------------------------------
'
GPCR$ = "(c) 1992-96 by H.-R.H. Wernli, CH-3952 Susten."
STCR$ = "Processed with " + VN$ + "; (c) H.-R.H.Wernli, CH-3952 Susten."
Nill$ = ""
InpPath$ = curdir$
OutPath$ = curdir$
'
Z% = fix(fre(-1)/1024)
Fatal$ = SLText$(57%) + str$(200-Z%) + "Kb"
if Z% < 200 then goto NoJob	'Needs about 324Kb
'
dim IOExt$(1:8)
IOExt$(1) = "LNX"
IOExt$(2) = "ALI"
IOExt$(3) = "ST4"
IOExt$(4) = "ASI"
IOExt$(5) = "BYT"
IOExt$(6) = "PIX"
IOExt$(7) = "ST6"
IOExt$(8) = "FIT"
dim FSize&(1:8)
FSize&(1) = 47520
FSize&(2) = 47520
FSize&(3) = 31872
FSize&(4) = 31872
FSize&(5) = 31680
FSize&(6) = 63367
FSize&(7) = 65535			'-- Fake size
FSize&(8) = 2880			'-- Record size
dim HSV%(1:4)				'-- Hot Spot Value
HSV%(1) = 32767
HSV%(2) = 16383
HSV%(3) =  8191
HSV%(4) =  4095
dim BatchI$(1:16)
dim BatchO$(1:16)
dim BatchF$(1:16)	'-- FITS only
dim Batch%(1:16)
dim FilPos&(1:16)	'-- FITS fileposition
dim Naxis%(1:16,0:6)	'-- 0,1,2,3: NAXIS, 4: BITPIX, 5: SIMPLE, 6: ST6
dim Flip%(1:16)
dim CCD%(1:165,1:192)
dim CCA%(1:31872)	'-- 192*166 to hold ST4 images
dim CCQ%(1:192,1:165)	'-- for QuickPix PIX
'
dim FH$(1:8)
FH$(1) = "SIMPLE  =" + space$(20) + "T"
FH$(2) = "BITPIX  =" + space$(19)
FH$(3) = "NAXIS   =" + space$(20) + "2"
FH$(4) = "NAXIS1  =" + space$(18) + "192"
FH$(5) = "NAXIS2  =" + space$(18) + "165"
FH$(6) = "ORIGIN    '" + STCR$ + "'"
FH$(7) = "DATE      '(mm-dd-yy) " + date$ + "'"
FH$(8) = "HISTORY   '"
'
'
' START OF PROGRAM
'============================================================================
'
' Setup of Filespec string-array, flags array and use sequence array
'
Virgin% = %True
'
'------------------------------ Log on --------------------------------------
'
print LogOn$(Version$,%Cyan,%Black)
print Describe$
'
'
'=============================-<Start of other Conversion Loop>-=============
do
  for N% = 1 to 16
    BatchI$(N%) = Nill$
    BatchO$(N%) = Nill$
    BatchF$(N%) = Nill$
    Batch%(N%)  = %False
    for F% = 0 to 6
      Naxis%(N%,F%) = 0
    next F%
    Flip%(N%) = 0
  next N%
  PixFlg% = %False
'
' Log On and input option gathering: 1...8
'
  print LogOn$(Version$,%Cyan,%Black);
  print Describe$
  Y% = csrlin
  locate Y%,1
  color %LCyan,%Black
  print Center$(SLText$(5%),80)
  color %Cyan,%Black
  call MLText(2%,15%,"tf")
'
'-|-------------------------------- Start of Input Option Loop --------------
  do
    InpOpt% = Prompt%(20%,Q$)
    if InpOpt% > -1 and InpOpt% < 9 then exit loop
    print Outch$(Q$)
  loop
  if InpOpt% = 0 then exit loop
  if InpOpt% > 5 then PixFlg% = %True else PixFlg% = %False
'
'.|................................ End of Input Option Loop ................
'
' Output option gathering 1...8
'
  print LogOn$(Version$,%Cyan,%Blue);
  print Describe$
  color %LCyan,%Black
  print Center$(SLText$(6%),80)
  color %Cyan,%Black
  call MLText(3%,19%,"tf")
'
'-|------------------------------ Start of Output Option Loop -------------
  do
    OutOpt% = Prompt%(61%,Q$)
    if OutOpt% > -1 and OutOpt% < 9 then exit loop
    print Outch$(Q$)
  loop
'
'.|.............................. End of Output Option Loop ...............
'
  if OutOpt% = 0 then exit loop
  if OutOpt% > 5 then PixFlg% = %True else PixFlg% = %False
'
'-- Prepare file extensions
'
  InpExt$ = IOExt$(InpOpt%)
  OutExt$ = IOExt$(OutOpt%)
'
'
'-- Get Output path
'
  print LogOn$(Version$,%Brown,%Black);
  print Describe$
  I% = %False
  gosub GetDirPath
  OutPath$ = UsePath$
  if Virgin% then InpPath$ = UsePath$	'-- only the first time
'
'-|------------------------- Start of Input Path Loop ----------------------
  do
    print LogOn$(Version$,%Green,%Black);
    I% = %True
    gosub GetDirPath			'-- Get input path
    InpPath$ = UsePath$
    A% = ListFiles%(InpPath$,InpExt$,English%)
    if A% then exit loop
    beep
    Fatal$ = SLText$(3%)
    if not Redo%(SLText$(39%)) then goto NoJob
  loop
'.|............................... End of Input Path Loop ...................
'
  Y? = csrlin
  if Y? > 21 then
    for N% = 1 to 7
      print
    next N%
  end if
  locate 21,1
'
'-- Input file gathering
'
  color %Black,%Cyan
  print Center$(SLText$(2%),80)
  color %Cyan,%Black
  Y% = csrlin
  for N% = 1 to 16
    locate Y%,1
    print space$(80);
    locate Y%,1
    A$ = IOFile$(English%,%False,SLText$(36%))
    if instr(A$,"END") then exit for
    BatchI$(N%) = InpPath$ + StripExt$(A$) + "." + InpExt$
    BatchO$(N%) = OutPath$ + StripExt$(A$) + "." + OutExt$
  next N%
  NoFiles% = N% - 1
'
'-- Filespec and filesize validification, construction of output filespecs
'
  print Logon$(Version$,%Cyan,%Blue);
  color %LCyan,%Black
  print Center$(SLText$(4%)+Space$(6)+"("+SLText$(16%)+")",80)
  for N% = 1 to NoFiles%
    A$ = left$(BatchO$(N%),len(BatchO$(N%))-1)
    If InpOpt% = OutOpt% then BatchO$(N%) = A$ + "C"
    color %White,%Black
    print "[";
    print using "##";N%;
    print "] ";
    color %Green,%Black
    print BatchI$(N%);" ";
    if not dir$(BatchI$(N%)) = "" then
      select case InpOpt%
        case 1, 2, 3, 4, 6
          open BatchI$(N%) for input as # %UnitAUX
          Z?? = lof(%UnitAUX)
        case 5
          open BatchI$(N%) for binary as # %UnitAUX
          get$ # %UnitAUX, 5,D$
          Z?? = asc(mid$(D$,4,1)) * 256 + asc(mid$(D$,3,1))
        case 7
          A%  = 0
          open BatchI$(N%) for input as # %UnitAUX
          line input# %UnitAUX, D$
          if  left$(D$,5) = "ST-6 "             then incr A%
          if right$(D$,7) = " Image" + chr$(10) then incr A%
          if instr(D$,"Compressed") then ST6C% = %True else ST6C% = %False
          if A% = 2 then Z?? = FSize&(InpOpt%) else Z?? = 0
        case 8
          open BatchI$(N%) for binary as # %UnitAUX
          A% = lof(%UnitAUX) mod FSize&(InpOpt%)
          if A% = 0 then Z?? = FSize&(InpOpt%) else Z?? = 0
          NA% = lof(%UnitAUX) \ FSize&(InpOpt%)
          if istrue NA% then
            BatchF$(N%) = StripExt$(BatchO$(N%)) + ".FIH"
            call FITHead(Naxis%(),BatchF$(),FilPos&(),N%)
          end if
      end select
      if Z?? = FSize&(InpOpt%) then
	color %LGreen,%Black
	print ""
	Batch%(N%) = %True
      else
	color %LRed,%Black
	print SLText$(18%);lof(1)
      end if
      close %UnitAUX
    else
      color %LRed,%Black
      print SLText$(19%)
      Batch%(N%) = %False
    end if
  next N%
'
'-- Moving filespecs such that invalid ones are skipped
'
  Cnt% = 0
  for N% = 1 to NoFiles%
    if Batch%(N%) then
      incr Cnt%
      BatchI$(Cnt%) = BatchI$(N%)
      BatchO$(Cnt%) = BatchO$(N%)
    end if
  next N%
  NoFiles% = Cnt%
  Q$ = KeyIn$
'
' We ask for each file whether additional processing is required
'
  call ClsTB
  print LogOn$(Version$,%White,%Red)
  color %LWhite,%Black
  call MLText(5%,80%,"cs")
  print
  color %LCyan,%Black
  print Center$(SLText$(29%),80)
  color %Cyan,%Black
  call MLText(4%,2%,"tf")
  Y% = csrlin - 1
  for N% = 1 to NoFiles%
    color %LGreen,%Black
    locate 23,1
    print Center$(BatchO$(N%),80);
    color %Yellow,%Black
    locate Y%,7
    print "?";
    locate Y%,7
    Q$ = KeyIn$
    A% = val(Q$)
    Flip%(N%) = A%
    A$ = left$(BatchO$(N%),len(BatchO$(N%))-1)
    select case A%
      case 1
        gosub CentOpt
        BatchO$(N%) = A$ + "1"
      case 2
        BatchO$(N%) = A$ + "2"
      case 3
        gosub CentOpt
        BatchO$(N%) = A$ + "3"
      case 4
        BatchO$(N%) = A$ + "M"
      case 5
        BatchO$(N%) = A$ + "U"
      case 6
        BatchO$(N%) = A$ + "F"
    end select
    locate 23,1
    call ClrLine
  next N%
'
' Verify to overwrite or skip existing files, if any - but only, if not
' Lynxx - Lynxx conversion
'
  print LogOn$(Version$,%White,%Red)
  color %Cyan,%Black
  print Center$(SLText$(11%),80)
  for N% = 1 to NoFiles%
    if not dir$(BatchO$(N%)) = "" then
      color %Green,%Black
      print BatchO$(N%);" ";
      color %LRed,%Black
      print SLText$(8%)
      if Confirm%(SLText$(9%)) then kill BatchO$(N%) else Batch%(N%) = %False
    end if
  next N%
  color %LWhite,%Black
'
' Now we can start doing the job. We select the input menu and for each one,
' do all output possibilities.
'
  print LogOn$(Version$,%White,%Blue)
  color %Cyan,%Black
  print SLText$(17%);
  Y? = csrlin
  X? = pos(Y?)
  for N% = 1 to NoFiles%
    locate Y?,X?
    color %LCyan,%Black
    print using "###";NoFiles%-N%+1
    color %Cyan,%Black
    print
    ST4Info$ = chr$(190) + space$(190) + chr$(0)
    if Batch%(N%) then
      call FillWC(CCD%(1,1),0%,%LMN)
      call FillWC(CCQ%(1,1),0%,%LMN)
      on InpOpt% gosub ILNX, ILNX, IST4, IST4, IBYT, IPIX, IST6, IFIT
    end if
    for F? = Y?+1 to 20
      locate F?,1
      call ClrLine
    next F?
  next N%
  locate Y?,X?
  color %LCyan,%Black
  print using "###";NoFiles%-N%+1
  color %Cyan,%Black
'
' Redo all with different input and output menus
'
  print
  Virgin% = %False
  print LogOn$(Version$,%White,%Blue)
  if not Confirm%(SLText$(7%)) then exit loop
loop
'
'::::::::::::::::::::::::::::::: End of other Conversion Loop :::::::::::::::
'
' That's it - we're going to log off
'
LogOff:
'
print LogOn$(Version$,%White,%Blue);
print Describe$
color %LWhite,%Green
A$ = VN$ + SLText$(30%)
print Center$(A$,80)
color %LWhite,%Red
A$ = VN$ + "; " + GPCR$
print Center$(A$,80)
color %White,%Black
locate 22,1,1,6,7
goto About
'
' Should an unrecoverable error occur, we exit program here
'
NoJob:
'
print
color %Yellow,%Red
A$ = " * " + SLText$(15%) + " - "
print Center$(A$ + Fatal$,80);
color %LWhite,%Red
print Center$(SLText$(31%) + VN$ + SLText$(32%),80);
color %LCyan,%Blue
print Center$(SLText$(33%),80);
color %White,%Black
locate 22,1,1,6,7
A$ = KeyIn$
goto About
'
'
About:
'
print "Info = F1 (2 sec)"
delay 2
if inkey$ = chr$(0,59) then
  print
  color %LWhite,%Black
  print "CCDCONVT.BAS Version";CCDConvtV%
  color %White,%Black
  print "CCDCPROC.INC Version";CCDCProcV%
  print "CCDCFUNC.INC Version";CCDCFuncV%
  print "CCDCSUB .INC Version";CCDCSubV%
  print "CCDCDAT .INC Version";CCDCDatV%
  print "CCDCC   .OBJ Version";CCDCCV%
  print "IOTEXT  .INC Version";IOTextV%
  print "ERROR3  .INC Version";Error3V%
  print "PB3GPUSE.PBU Version";PB3GPUseV%
  print "ARRAYEXT.PBL Version";ArrayExtV%
  print "CLRSCRN .PBL Version";ClrScrnV%
  print "RLCEGEN .OBJ Version";RlceGenV%
  print "PBSTACK .OBJ Version 100"
  print "PowerBASIC  Revision";PBVer%(SV$);SV$
  print
  color %LWhite,%Blue
  call HoroSoftLogo
  color %White,%Black
  while not instat : wend
  Q$ = inkey$
  call ClsCE
end if
cls
system
end
'
'
'===================================-<920912>-=========-<HoroSoft>-==========
'
