'960210 1454
'****************************************************************************
'* PROGRAM NAME: CCDCPROC.INC   DATE: 13. 9.1992   INCLUDE IN: CCDCONVT.BAS *
'* LANGUAGE: Power BASIC 3.20   Robert S.Zale                               *
'* PURPOSE: Collection of procedures used in CCDCONVT.BAS                   *
'* INCLUDE FILES: none - though they must be included/linked by main        *
'* INPUT FILES: none - files accessed must be opened by main                *
'* OUTPUT FILES: none - files written to must be opened by main             *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.07: 10-FEB-96 Recompiled foth PB 3.20                         *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.06: 25-JUN-93 Added FITS format                               *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.05:  1-JUN-93 Added Version CCDCPROCV% = 105                  *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.04: 31-DEC-93 BYT Files are all RLCE encoded                  *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.03:  1-JAN-93 Introduced function TestZero%.                  *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.02: 21-SEP-92 Changed Synt-Lynxx routine. Synthesized Lynxx   *
'*                pictures showed streakes when stretched in VGA mode with  *
'*                Lynxx-program. Now, the synthesized image matches the     *
'*                original one, even the histogram.                         *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.01: 20-SEP-92 CCDPROC was modified. Overwholed to put all     *
'*                texts into data area in main program.                     *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.00: 13-SEP-92 Subset of CCDPROC.INC                           *
'* ------------------------------------------------------------------------ *
'* COMMENTS:                                                                *
'* ======================================================================== *
'* AUTHOR: Hans-Ruedi H. Wernli, Pletschgasse, CH-3952 Susten, Switzerland. *
'****************************************************************************
'
'
CCDCProcV% = 107
'
'
' PROCEDURE  checks whether lowest value is still positive. If so,
'            multiplies by two until highest value is between 32767 and
'            16384 (14-bit value)
'----------------------------------------------------------------------------
'
sub MovMax(CCD%(2))
  local C%,M%
  if MinWrd%(CCD%(1,1),%LMN) > -1 then
    C% = 0
    M% = MaxWrd%(CCD%(1,1),%LMN)
    do
      if M% > 16383 then exit loop
      M% = M% * 2
      incr C%
    loop
    if C% > 0 then call ShlMWA(CCD%(1,1),C%,%LMN)
  end if
end sub
'
'
' PROCEDURE  checks whether lowest value is positive. If so, divides all
'            values, until highest value is lower than A%
'----------------------------------------------------------------------------
'
sub MovBak(CCD%(2),A%)
  local C%,N%,M%
  if MinWrd%(CCD%(1,1),%LMN) > -1 then
    C% = 0
    M% = MaxWrd%(CCD%(1,1),%LMN)
    do
      if M% < A% then exit loop
      M% = M% \ 2
      incr C%
    loop
    if C% > 0 then call ShrMWA(CCD%(1,1),C%,%LMN)
  end if
end sub
'
'
' PROCEDURE Flips or rotates image according to selected option
'----------------------------------------------------------------------------
'
sub FlipIt(CCD%(2),CCQ%(2),O%)
  local A%
  A% = 14
  select case O%
    case 1, 3, > 512, > 256
      if O% > 512 then
	O% = O% - 512
	A% = 28
      end if
      if O% > 256 then
	O% = O% - 256
	A% = 1
      end if
      call MovLft(CCD%(1,1),A%)
  end select
  if not O% = 2 then call XDimWA(CCQ%(1,1),CCD%(1,1),192%,165%)
  select case O%
    case 1
      call FlipXY(CCQ%(1,1),3%)
    case 2
      call RoTwic(CCD%(165,192),CCD%(1,1))
    case 3
      call FlipXY(CCQ%(1,1),3%)
      call FlipXY(CCQ%(1,1),3%)
      call FlipXY(CCQ%(1,1),3%)
    case 4
      call FlipXY(CCQ%(1,1),1%)
    case 5
      call FlipXY(CCQ%(1,1),2%)
    case 6
      call FlipXY(CCQ%(1,1),1%)
      call FlipXY(CCQ%(1,1),2%)
  end select
  if not O% = 2 then call XDimWA(CCD%(1,1),CCQ%(1,1),165%,192%)
end sub
'
'
' PROCEDURE finds Hot Spots in ST-6 image and prompts for removing
'----------------------------------------------------------------------------
'
sub HotSpots(CCD%(2),HSV%())
  local A%,F%,H1%,H2%,H3%,H4%,Y%,Q$
  H1% = HotSpt%(CCD%(1,1),32768??)
  H2% = HotSpt%(CCD%(1,1),16384??)
  H3% = HotSpt%(CCD%(1,1), 8196??)
  H4% = HotSpt%(CCD%(1,1), 4096??)
  if istrue (H1% + H2% + H3% + H4%) then
    Y% = csrlin
    print
    color %Cyan,%Black
    print SLText$(53%);" 32767:";
    color %LCyan,%Black
    print using "####";H1%;
    color %Cyan,%Black
    print SLText$(54%);"1]"
    print SLText$(53%);" 16383:";
    color %LCyan,%Black
    print using "####";H2%;
    color %Cyan,%Black
    print SLText$(54%);"2]"
    print SLText$(53%);"  8191:";
    color %LCyan,%Black
    print using "####";H3%;
    color %Cyan,%Black
    print SLText$(54%);"3]"
    print SLText$(53%);"  4095:";
    color %LCyan,%Black
    print using "####";H4%;
    color %Cyan,%Black
    print SLText$(54%);"4]"
    color %LCyan,%Black
    print SLText$(55%);" (0...4): ";
    color %Yellow,%Black
    line input Q$
    color %White,%Black
    A% = IntLmt%(0%,4%,val(Q$))
    for F% = Y% +1  to Y% + 5
      locate F%,1
      call ClrLine
    next F%
    locate Y%,1
    if A% then call LmtHWC(CCD%(1,1),HSV%(A%),%LMN)
  end if
end sub
'
'
' PROCEDURE writes FITS header to file.FIH and returns in Naxis%(x,I%):
'           I%=0: NAXIS, 1: NAXIS1, 2: NAXIS2, 3:NAXIS3, 4: BITPIX
'           I%=5: SIMPLE true or false. FilPos& returns the file position
'           where data starts. The header-file.FIH can be deleted by the
'           user.
'----------------------------------------------------------------------------
'
sub FITHead(Naxis%(2),BatchF$(),FilPos&(),I%)
  local N%,P%,C$,D$,S$
  S$ = "ST-6 camera software from SBIG"
  Naxis%(I%,6) = %False
  if FileHere%(BatchF$(I%)) then kill BatchF$(I%)
  open BatchF$(I%) for output as # %UnitFIT
  get$ # %UnitAUX, 2880, D$
  P% = 2
  Naxis%(I%,5) = SimpF%(left$(D$,80))
  Naxis%(I%,4) = ValuF%(mid$(D$, 81,80),"BITPIX  =")
  Naxis%(I%,0) = ValuF%(mid$(D$,161,80),"NAXIS   =")
  if Naxis%(I%,0) > 0 then
    Naxis%(I%,1) = ValuF%(mid$(D$,241,80),"NAXIS1  =")
    incr P%
  end if
  if Naxis%(I%,0) > 1 then
    Naxis%(I%,2) = ValuF%(mid$(D$,321,80),"NAXIS2  =")
    incr P%
  end if
  if Naxis%(I%,0) > 2 then
    Naxis%(I%,3) = ValuF%(mid$(D$,401,80),"NAXIS3  =")
    incr P%
  end if
  for N% = 0 to P%
    print# %UnitFIT, rtrim$(mid$(D$,N%*80+1,80))
  next N%
  do
    for N% = P%+1 to 35
      P% = N% * 80 + 1
      C$ = mid$(D$,P%,80)
      if not left$(C$,1) = " " then
        print# %UnitFIT, rtrim$(C$)
        if istrue instr(C$,S$) then Naxis%(I%,6) = %True
        if left$(C$,3) = "END" then exit for
      end if
    next N%
    if left$(C$,3) = "END" then exit loop
    P% = 0
    get$ # %UnitAUX, 2880, D$
  loop
  FilPos&(I%) = loc(%UnitAUX)
  close %UnitFIT
end sub
'
'
' PROCEDURE displays FITS-Head
'-----------------------------------------------------------------------------
'
sub FHeadD(V$,O$,B$,Naxis%(2),I%)
  local N%,C$,D$
  print LogOn$(V$,%White,%Brown)
  print B$
  if Naxis%(I%,0) = 3 then print SLText$(66%);Naxis%(I%,3);SLText$(67%);"."
  if Confirm%(SLText$(59%)) then
    open O$ for input as # %UnitFIT
    while not eof(%UnitFIT)
      line input# %UnitFIT, D$
      color %Green,%Black
      print left$(D$,8);
      color %LGreen,%Black
      C$ = mid$(D$,9)
      if len(D$) > 79 then print C$; else print C$
      incr N%
      if N% = 17 then
        if isfalse Confirm%(SLText$(62%)) then exit loop
        N% = 1
      end if
    wend
    close %UnitFIT
  end if
  color %Cyan,%Black
  print SLText$(60%)
  if isfalse Confirm%(SLText$(61%)) and istrue FileHere%(O$) then kill O$
  print LogOn$(V$,%White,%Blue)
end sub
'
'
' PROCEDURE gets FITS-data. I% is index (N%), FP& the filepointer,
'           A1% start column and A2% end column, both option base 0
'----------------------------------------------------------------------------
'
sub GetLFit(CCD%(2),CCQ%(2),Naxis%(2),FP&,A1%,A2%,I%)
  local A%,B%,N%,L0%,L1%,R%,O%,Z&,D$,X%,Y%
  B% = 1
  Z& = A2% * Naxis%(I%,1) + A1%
  FP& = FP& + Z&
  if Naxis%(I%,4) = 16 then
    FP& = FP& + Z&
    B% = 2
  end if
  if Naxis%(I%,1) > 192 then
    L1% = (Naxis%(I%,1) - 192) * B%
    L0% = 192 * B%
  else
    L1% = 0
    L0% = Naxis%(I%,1) * B%
  end if
  if Naxis%(I%,2) > 165 then R% = 165 else R% = Naxis%(I%,2)
  print
  color %Cyan,%Black
  print SLText$(72%);
  Y% = csrlin
  X% = pos(Y%)
  color %LCyan,%Black
  for N% = 1 to R%
    locate Y%,X%
    print using "####";N%;
    seek # %UnitAUX, FP&
    get$ # %UnitAUX, L0%, D$
    FP& = FP& + L0% + L1%
    O% = (N%-1) * 192 * B%
    call StrVar(CCQ%(1,1),L0%,O%,D$)
  next N%
  if Naxis%(I%,4) = 8 then
    call CpyB2W(CCD%(1,1),CCQ%(1,1),%LMN)
  else
    call CpyW2W(CCD%(1,1),CCQ%(1,1),%LMN)
  end if
  print
  color %White,%Black
end sub
'
'
'===============================-<920913>-============-<HoroSoft>-===========
