'960411 1801
'****************************************************************************
'* PROGRAM NAME: CCDCSUB .INC   DATE: 18.10.1992   INCLUDE IN: CCDCONVT.BAS *
'* LANGUAGE: Power BASIC 3.20   Robert S.Zale                               *
'* PURPOSE: Subroutines for CCDCONVT                                        *
'* INCLUDE FILES: none.                                                     *
'* OBJECT FILES:  none.                                                     *
'* INPUT FILES:   none.                                                     *
'* OUTPUT FILES:  none.                                                     *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.07 11-APR-96 Exposure time for ST4 corrected from ms to s/100 *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.06  8-APR-96 removed HoroSoft Logo, now in CLRSCRN.PBL        *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.05 10-FEB-96 Added Initialization for HoroSoft Logo           *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.04 25-JUN-94 Added FITS format                                *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.03  1-JUN-94 Added compression percentage for OBYT and        *
'*               Version number as CCDCSUBV% = 103                          *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.02 18-JAN-94 Added ST6 file conversion                        *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.01 31-DEC-93 BYT Files are all RLCE encoded                   *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.00 21-MAY-93 Extracted from CCDCONVT.BAS.                     *
'* ------------------------------------------------------------------------ *
'* COMMENTS:                                                                *
'* ======================================================================== *
'* AUTHOR: Hans-Ruedi H. Wernli, Pletschgasse, CH-3952 Susten, Switzerland. *
'****************************************************************************
'
'
CCDCSUBV% = 107
'
'
goto CCDCSUB
'
'
' SUBROUTINE returns in A% a true flag if no Path/Drive error occured.
'----------------------------------------------------------------------------
'
ValidatePath:
'
E% = 0
D$ = curdir$
on error goto VPERR
chdir left$(UsePath$,len(UsePath$)-1)
chdir D$
VPRES:
if E% = 0 then A% = %True else A% = %False
on error goto ErrorHandling
return
VPERR:
E% = err
resume VPRES
'
'
' SUBROUTINE returns validated path. I% = 1 for Input, 2 for output
'----------------------------------------------------------------------------
'
GetDirPath:
'
do
  if I% then
    UsePath$ = IOPath$(SLText$(1%),InpPath$,SLText$(36%))
  else
    UsePath$ = IOPath$(SLText$(10%),OutPath$,SLText$(36%))
  end if
  gosub ValidatePath
  if A% then exit loop
  if not Redo%(SLText$(37%)) then
    Fatal$ = SLText$(38%)
    return NoJob
  end if
  locate csrlin-1,1
loop
return
'
'
' Centering Option for Rotation - Since image must be squarred from 192 X 165
'   to 165 X 165, we ask for left, right or centering of image. If left adjust
'   is desired, we add 256 to Flip%, if right is desired, 512. No offset is
'   added for centering of image.
'-----------------------------------------------------------------------------
'
CentOpt:
'
locate 22,1
call ClrLine
color %LCyan,%Black
print Center$(SLText$(35%),79);
Q$ = KeyIn$
if Q$ = "L" then Flip%(N%) = Flip%(N%) + 256
if Q$ = "R" then Flip%(N%) = Flip%(N%) + 512
locate 22,1
call ClrLine
return
'
'
' > > >  C O N V E R T E R S  < < <
'
' INTERFACE: After read, data is in CCD%(1:165,1:192), i.e. all columns for
'            each line. Column/row swapping is done on output converters.
'            Element values are set to [0;4095] and reduced or expanded
'            later when appropriate.
'            If input and output filetypes are the same and no processing
'            is required, file is copied using the DOS COPY command.
'            Filename will become filename.xxC
'----------------------------------------------------------------------------
'
' Input-Lynxx - Lynxx files have 192 columns for one row and there are 165
'   Two words are lines. Two words are compressed into three bytes.
'   Processing is done here.
'   Range [0;4095] --> [0;4095]
'-----------------------------------------------------------------------------
'
ILNX:
'
Originator$ = "Lynxx Img. "
FitsHist$ = "Spectra Source Instruments PC-Lynxx"
if TestZero%(Flip%(N%),OutOpt%,1%,2%) then
  A$ = "COPY " + BatchI$(N%) + " " + BatchO$(N%)
  shell A$
else
  open BatchI$(N%) for binary as # %UnitAUX
  print BatchI$(N%);SLText$(12%)
  Handle% = fileattr(%UnitAUX,2)
  call DecLnx(Handle%,CCD%(1,1))
  close %UnitAUX
  Fatal$ = SLText$(25%)
  if not Handle% then goto WarnIO
  call MovMax(CCD%())				'new
  call FlipIt(CCD%(),CCQ%(),Flip%(N%))
  mid$(ST4Info$,2,11) = Originator$
  on OutOpt% gosub OLNX, OLNX, OST4, OST4, OBYT, OPIX, OST6, OFIT
end if
return
'
'
' Input-ST-4 - ST4 files have 192 columns for a line and 165 lines. Each
'   pixel uses a byte. The 166th line holds information.
'   Range [0;255] --> [0;4095]
'-----------------------------------------------------------------------------
'
IST4:
'
Originator$ = "ST-4 Img.  "
FitsHist$ = "Santa Barbara Instrument Group ST-4"
if TestZero%(Flip%(N%),OutOpt%,3%,4%) then
  A$ = "COPY " + BatchI$(N%) + " " + BatchO$(N%)
  shell A$
else
  open BatchI$(N%) for binary as # %UnitAUX
  print BatchI$(N%);SLText$(12%)
  R$ = space$(192)
  Handle% = fileattr(%UnitAUX,2)
  call EDCST4(%DOSRead,R$,Handle%,CCA%(1))
  close %UnitAUX
  Fatal$ = SLText$(26%)
  if not Handle% then goto WarnIO
  call CpyB2W(CCD%(1,1),CCA%(1),%LMN)
  color %LGreen,%Black
  print mid$(R$,2,78)
  color %Green,%Black
  print "Exposure time (s/100): ";
  color %LGreen,%Black
  print mid$(R$,80,10)
  color %Green,%Black
  print "Focal length (inch): ";
  color %LGreen,%Black
  print mid$(R$,90,10)
  color %Green,%Black
  print "Apperture (inch):    ";
  color %LGreen,%Black
  print mid$(R$,100,10)
  color %Green,%Black
  print "Calibration factor (1): ";
  color %LGreen,%Black
  print mid$(R$,110,10)
  color %LGreen,%Black
  print mid$(R$,120,72)
  A$ = left$(R$,119) + STCR$
  ST4Info$ = A$
  call MovMax(CCD%())
  call FlipIt(CCD%(),CCQ%(),Flip%(N%))
  on OutOpt% gosub OLNX, OLNX, OST4, OST4, OBYT, OPIX, OST6, OFIT
end if
return
'
'
' Input-Byte - Byte files have 165 lines with 192 columns each. There is one
'   byte for each pixel, compressed using RLCE (Run Length Count Encoding).
'   Range [0;255] --> [0;4095]
'----------------------------------------------------------------------------
'
IBYT:
'
Originator$ = "Byte Img.  "
FitsHist$ = "HoroSoft CCDTOOLS/CCDUTIL Byte"
if TestZero%(Flip%(N%),OutOpt%,5%,5%) then
  A$ = "COPY " + BatchI$(N%) + " " + BatchO$(N%)
  shell A$
else
  open BatchI$(N%) for binary as # %UnitAUX
  Handle% = fileattr(%UnitAUX,2)
  Fatal$ = SLText$(43%)
  print SLText$(20);" ";BatchI$(N%)
  Z% = RdBie%(CCQ%(1,1),Handle%,31685??)
  close %UnitAUX
  if Z% then goto WarnIO
  print SLText$(44%);
  S% = DecBie%(CCA%(1),CCQ%(1,1))
  Fatal$ = SLText$(45%) + str$(S%)
  if S% then
    print
    gosub WarnIO
  else
    print " "
  end if
  call CpyB2W(CCQ%(1,1),CCA%(1),%LMN)
  call XDimWA(CCD%(1,1),CCQ%(1,1),165%,192%)
  call MovMax(CCD%())
  call FlipIt(CCD%(),CCQ%(),Flip%(N%))
  color %White,%Black
  mid$(ST4Info$,2,11) = Originator$
  on OutOpt% gosub OLNX, OLNX, OST4, OST4, OBYT, OPIX, OST6, OFIT
end if
return
'
'
' Input-Pixel - Pixel files have 192 columns with 165 lines each. Each pixel
'   uses one word.
'   Range [0;4095] --> [0;4095]
'-----------------------------------------------------------------------------
'
IPIX:
'
Originator$ = "Pixel Img. "
FitsHist$ = "Electrim Pixel"
if TestZero%(Flip%(N%),OutOpt%,6%,6%) then
  A$ = "COPY " + BatchI$(N%) + " " + BatchO$(N%)
  shell A$
else
  open BatchI$(N%) for binary as # %UnitAUX
  print SLText$(41%);BatchI$(N%);
  Handle% = fileattr(%UnitAUX,2)
  seek # %UnitAUX,7
  call EDCPix(%DOSRead,Handle%,CCD%(1,1))
  close %UnitAUX
  Fatal$ = SLText$(42%)
  if not Handle% then goto WarnIO
  print " "
  call MovMax(CCD%())
  call FlipIt(CCD%(),CCQ%(),Flip%(N%))
  color %White,%Black
  mid$(ST4Info$,2,11) = Originator$
  on OutOpt% gosub OLNX, OLNX, OST4, OST4, OBYT, OPIX, OST6, OFIT
end if
return
'
'
' Input-ST6 - ST6 files have 375 columns and 242 lines. Each pixel has one
'   word. A head with 1024 byte preceeds the image data. Data may be
'   compressed using delta line compression, or be uncompressed.
'   Since the image is too large for this converter, the size may be
'   reduced appropriately or the size halfed by binning.
'   Range [0;65535] --> [0;4095]
'-----------------------------------------------------------------------------
'
IST6:
'
Originator$ = "ST-6 Img.  "
FitsHist$ = "Santa Barbara Instrument Group ST-6"
if TestZero%(Flip%(N%),OutOpt%,7%,7%) then
  A$ = "COPY " + BatchI$(N%) + " " + BatchO$(N%)
  shell A$
else
  print
  Y% = csrlin
  X% = pos(Y%)
  color %Cyan,%Black
  call MLText(7%,80%,"cs")
  color %LCyan,%Black
  print
  print BatchI$(N%)
  if Confirm%(SLText$(48%)) then
    print
    Shrink% = %False
    print SLText$(49%);
    line input Q$
    LCS% = IntLmt%(1%,183%,val(Q$))
    print SLText$(50%);
    line input Q$
    TLS% = IntLmt%(1%,77%,val(Q$))
  else
    Shrink% = %True
    LCS% = 1				'-- Left Column Start
    TLS% = 1				'-- Top Line Start
    Originator$ = "ST-6 Binned"
    FitsHist$ = "Santa Barbara Instrument Group ST-6 (binned)"
  end if
  for F% = Y% to 24
    locate F%,1
    call ClrLine
  next F%
  locate Y%,X%
  color %Cyan,%Black
  open BatchI$(N%) for binary as # %UnitAUX
  Handle% = fileattr(%UnitAUX,2)
  seek # %UnitAUX, 2048
  print BatchI$(N%);SLText$(12)
  Fatal$ = SLText$(51%)
  color %Cyan,%Black
  print SLText$(52%);
  Y% = csrlin
  X% = pos(Y%)
  color %LCyan,%Black
  CDO?? = 0
  if Shrink% then
    for F% = 1 to 242 step 2
      locate Y%,X%
      print using "####";F%;
      if not ST6C% then
        if not TwoLU%(CCQ%(1,1),Handle%) then goto WarnIO
      else
        if not TwoLC%(CCQ%(1,1),CCA%(1),Handle%) then goto WarnIO
      end if
      call TwoBin(CCD%(1,1),CCQ%(1,1),CDO??)
      CDO?? = CDO?? + 384
    next F%
  else
    if TLS% > 1 then
      for F% = 1 to TLS% - 1 step 2
        locate Y%,X%
        print using "####";F%;
        if not ST6C% then
          if not TwoLU%(CCQ%(1,1),Handle%) then goto WarnIO
        else
          if not TwoLC%(CCQ%(1,1),CCA%(1),Handle%) then goto WarnIO
        end if
      next F%
    end if
    QO??  = (LCS%-1) * 2
    for F% = TLS% to TLS% + 164 step 2
      locate Y%,X%
      print using "####";F%;
      if not ST6C% then
        if not TwoLU%(CCQ%(1,1),Handle%) then goto WarnIO
      else
        if not TwoLC%(CCQ%(1,1),CCA%(1),Handle%) then goto WarnIO
      end if
      call TwoGet(CCD%(1,1),CCQ%(1,1),CDO??,QO??)
      CDO?? = CDO?? + 768
    next F%
  end if
  close %UnitAUX
  call HotSpots(CCD%(),HSV%())
  call MovMax(CCD%())
  print " "
  call FlipIt(CCD%(),CCQ%(),Flip%(N%))
  color %White,%Black
  mid$(ST4Info$,2,11) = Originator$
  on OutOpt% gosub OLNX, OLNX, OST4, OST4, OBYT, OPIX, OST6, OFIT
end if
return
'
'
' Input-FITS - Input Flexible Image Transport System files
'----------------------------------------------------------------------------
'
IFIT:
'
Originator$ = "FITS Image "
FitsHist$ = "Flexible Image Transport System FITS"
if TestZero%(Flip%(N%),OutOpt%,8%,8%) then
  A$ = "COPY " + BatchI$(N%) + " " + BatchO$(N%)
  shell A$
else
  mid$(ST4Info$,2,11) = Originator$
  color %Cyan,%Black
  print BatchI$(N%)
  print
  if isfalse Naxis%(N%,5) then			'SIMPLE=F
    call MLText(6%,80%,"cs")
    A% = Confirm%(SLText$(63%))
    call FHeadD(Version$,BatchF$(N%),BatchI$(N%),Naxis%(),N%)
    return
  end if
  select case Naxis%(N%,4)			'BITPIX
    case 8, 16
      A$ = ""
    case 32
      call MLText(10%,80%,"cs")
      A% = Confirm%(SLText$(63%))
      call FHeadD(Version$,BatchF$(N%),BatchI$(N%),Naxis%(),N%)
      return
    case else
      call MLText(8%,1%,"ts")
      print Naxis%(N%,4);"bit"
      A% = Confirm%(SLText$(63%))
      call FHeadD(Version$,BatchF$(N%),BatchI$(N%),Naxis%(),N%)
      return
  end select
  if Naxis%(N%,0) = 0 then			'NAXIS=0
    print Center$(SLText$(64%),80);
    A% = Confirm%(SLText$(63%))
    call FHeadD(Version$,BatchF$(N%),BatchI$(N%),Naxis%(),N%)
    return
  end if
  open BatchI$(N%) for binary as # %UnitAUX
  seek # %UnitAUX, FilPos&(N%)
  Handle% = fileattr(%UnitAUX,2)
  select case Naxis%(N%,0)
    case 1					'NAXIS=1
      call MLText(9%,1%,"ts")
      print Naxis%(N%,1)
      A% = Confirm%(SLText$(63%))
      call FHeadD(Version$,BatchF$(N%),BatchI$(N%),Naxis%(),N%)
      if Naxis%(N%,1) > 31680 then Z% = 31680 else Z% = Naxis%(N%,1)
      select case Naxis%(N%,4)
        case 8					'NAXIS=1, BITPIX=8
          A% = RdFit%(0%,Handle%,Z%,CCA%(1))
          Fatal$ = SLText$(25%)
          if istrue A% then goto WarnIO
          call CpyB2W(CCD%(1,1),CCA%(1),%LMN)
        case 16					'NAXIS=1, BITPIX=16
          A% = RdFit%(1%,Handle%,Z%,CCD%(1,1))
          Fatal$ = SLText$(25%)
          if istrue A% then goto WarnIO
      end select
    case 2                                      'NAXIS=2
      call FHeadD(Version$,BatchF$(N%),BatchI$(N%),Naxis%(),N%)
    case 3					'NAXIS=3
      call FHeadD(Version$,BatchF$(N%),BatchI$(N%),Naxis%(),N%)
      color %LCyan,%Black
      print SLText$(68%);" 1 ...";Naxis%(N%,3);"? ";
      color %Yellow,%Black
      line input Q$
      Q% = IntLmt%(1,Naxis%(N%,3),val(Q$))
      Z& = Naxis%(N%,1) * Naxis%(N%,2) * (Q%-1)
      FilPos&(N%) = FilPos&(N%) + Z&
      if Naxis%(N%,4) = 16 then FilPos&(N%) = FilPos&(N%) + Z&
      Q$ = remove$(str$(Q%)," ")		'-- creates FxT extension
      Q% = len(BatchO$(N%)) - 1
      mid$(BatchO$(N%),Q%,1) = Q$
      print LogOn$(Version$,%White,%Blue)
    case else					'NAXIS > 3
      call MLText(11%,1%,"cf")
      A% = Confirm%(SLText$(63%))
      call FHeadD(Version$,BatchF$(N%),BatchI$(N%),Naxis%(),N%)
      return
  end select
  if Naxis%(N%,0) > 1 then
    AxSq& = Naxis%(N%,1) * Naxis%(N%,2)
    if AxSq& > 31680 then
      color %Cyan,%Black
      call MLText(12%,79%,"cf")
      print
      print SLText$(69%);" ";
      color %LCyan,%Black
      print Naxis%(N%,1)
      color %Cyan,%Black
      print SLText$(70%);" ";
      color %LCyan,%Black
      print Naxis%(N%,2)
      print
      print
      if Naxis%(N%,1) > 192 then
        A1% = RowCol%(Naxis%(N%,1),192%,1%)
      end if
      if Naxis%(N%,2) > 165 then
        A2% = RowCol%(Naxis%(N%,2),165%,2%)
      end if
    else
      A1% = 0
      A2% = 0
    end if
    call GetLFit(CCD%(),CCQ%(),Naxis%(),FilPos&(N%),A1%,A2%,N%)
  end if
  if Naxis%(N%,4) = 16 then
    call MotInt(CCD%(1,1),%LMN)
    if istrue Naxis%(N%,6) then
      call SubCWL(CCD%(1,1),32768??,0%,%LMN)
    else
      if MinWrd%(CCD%(1,1),%LMN) < 0 then call AbsIAH(CCD%(1,1),%LMN)
    end if
  end if
  close %UnitAUX
  call MovMax(CCD%())
  call FlipIt(CCD%(),CCQ%(),Flip%(N%))
  color %White,%Black
  mid$(ST4Info$,2,11) = Originator$
  on OutOpt% gosub OLNX, OLNX, OST4, OST4, OBYT, OPIX, OST6, OFIT
end if
return
'
'
' Output-Lynx - Write a Lynxx file.
'   Range [0;4095] --> [0;4095]
'-----------------------------------------------------------------------------
'
OLNX:
'
call MovBak(CCD%(),4096%)
if FileHere%(BatchO$(N%)) then kill BatchO$(N%)
open BatchO$(N%) for output as # %UnitAUX
print BatchO$(N%);SLText$(13%)
Handle% = fileattr(%UnitAUX,2)
call EncLnx(Handle%,CCD%(1,1))
close %UnitAUX
Fatal$ = SLText$(23%)
if not Handle% then goto WarnIO
return
'
'
' Output-ST-4 - Write an ST4 file.
'   Range [0;4095] --> [0;255]
'-----------------------------------------------------------------------------
'
OST4:
'
call MovBak(CCD%(),256%)
call CpyW2B(CCA%(1),CCD%(1,1),%LMN)
if FileHere%(BatchO$(N%)) then kill BatchO$(N%)
open BatchO$(N%) for output as # %UnitAUX
print BatchO$(N%);SLText$(13%)
mid$(ST4Info$,120,72) = STCR$
mid$(ST4Info$,120,72) = STCR$
Handle% = fileattr(%UnitAUX,2)
call EDCST4(%DOSWrite,ST4Info$,Handle%,CCA%(1))
close %UnitAUX
Fatal$ = SLText$(24%)
if not Handle% then goto WarnIO
return
'
'
' Output Byte - Write a Byte file.
'   Range [0;4095] --> [0;255]
'-----------------------------------------------------------------------------
'
OBYT:
'
call MovBak(CCD%(),256%)
call XDimWA(CCQ%(1,1),CCD%(1,1),192%,165%)
call CpyW2B(CCA%(1),CCQ%(1,1),%LMN)
print SLText$(46%);
Z% = EncBie%(CCQ%(1,1),CCA%(1),31680??)
Z! = Z% * 100 / 31680
print " (";remove$(str$(fix(Z!))," ");"%) "
if FileHere%(BatchO$(N%)) then kill BatchO$(N%)
open BatchO$(N%) for binary as # %UnitAUX
Handle% = fileattr(%UnitAUX,2)
Fatal$ = SLText$(47%)
Z% = WrBie%(CCQ%(1,1),Handle%)
close %UnitAUX
if Z% then goto WarnIO
return
'
'
' Output Pixel Word - Write a Pixel file.
'   Range [0;4095] --> [0;4095]
'-----------------------------------------------------------------------------
'
OPIX:
'
call MovBak(CCD%(),4096%)
if FileHere%(BatchO$(N%)) then kill BatchO$(N%)
open BatchO$(N%) for binary as # %UnitAUX
put$ # %UnitAUX, chr$(253,118,144,0,0,128,247)
Handle% = fileattr(%UnitAUX,2)
call EDCPix(%DOSWrite,Handle%,CCD%(1,1))
close %UnitAUX
Fatal$ = SLText$(34%)
if not Handle% then goto WarnIO
print " "
color %White,%Black
return
'
'
' Output ST-6 - write a ST6 file.
'   Range [0;4095] --> [0;65535]
'-----------------------------------------------------------------------------
'
OST6:
'
Fatal$ = SLText$(56%)
if FileHere%(BatchO$(N%)) then kill BatchO$(N%)
open BatchO$(N%) for binary as # %UnitAUX
Handle% = fileattr(%UnitAUX,2)
put$ # %UnitAUX, ST6Head$(VN$,Originator$)
color %Cyan,%Black
print SLText$(46%);" (242):";
Y% = csrlin
X% = pos(Y%)
color %LCyan,%Black
P% = 0
for F% = 1 to 165
  locate Y%,X%
  print using "####";F%;
  if not Delta%(CCQ%(1,1),CCD%(1,1),Handle%,P%) then goto WarnIO
  P% = P% + 384
next F%
D$ = chr$(121,1,0,0,128) + string$(374,chr$(0))
for F% = 166 to 242
  locate Y%,X%
  print using "####";F%;
  put$ # %UnitAUX, D$
next F%
close %UnitAUX
D$ = ""
print " "
color %White,%Black
return
'
'
' Output-FIT - Output Flexible Image Transport System file
'----------------------------------------------------------------------------
'
OFIT:
'
if (InpOpt% > 2) and (InpOpt% < 6) then
  A$ = " 8"
  SF% = 0
else
  A$ = "16"
  SF% = 1
end if
C$ = ""
for F% = 1 to 8
  Z$ = Nill$
  if F% = 2 then Z$ = A$
  if F% = 8 then Z$ = SLText$(73%) + FitsHist$ + "-Format'"
  C$ = C$ + left$(FH$(F%)+Z$+space$(80),80)
next F%
D$ = left$(C$+"END"+space$(2880),2880)
C$ = Nill$
if FileHere%(BatchO$(N%)) then kill BatchO$(N%)
open BatchO$(N%) for binary as # %UnitAUX
Handle% = fileattr(%UnitAUX,2)
put$ # %UnitAUX, D$
D$ = Nill$
Fatal$ = SLText$(74%)
if istrue SF% then
  call MotInt(CCD%(1,1),%LMN)
  Z% = WrFits%(CCD%(1,1),Handle%,SF%)
else
  call MovBak(CCD%(),256%)
  call CpyW2B(CCA%(1),CCD%(1,1),%LMN)
  Z% = WrFits%(CCA%(1),Handle%,SF%)
end if
close %UnitAUX
if istrue Z% then goto WarnIO
return
'
'
' Byte, Lynxx and ST4 direct DOS read or write error handler
'----------------------------------------------------------------------------
'
WarnIO:
'
beep
color %Yellow,%Red
print Center$(Fatal$,79)
color %White,%Black
A$ = KeyIn$
return
'
'
CCDCSUB:
'
'===================================-<930521>-=========-<HoroSoft>-==========
'
