'970413 1838
'****************************************************************************
'* PROGRAM NAME: CCDFFUNC.INC   DATE: 10. 2.1996   INCLUDE IN: CCDFITS .BAS *
'* LANGUAGE: Power BASIC 3.20   Robert S.Zale                               *
'* PURPOSE: Collection of functions used in CCDFITS.BAS                     *
'* INCLUDE FILES: none - though they must be included/linked by main        *
'* INPUT FILES: none - files accessed must be opened by main                *
'* OUTPUT FILES: none - files written to must be opened by main             *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.16: 13-APR-97 Added Starlight Xpress and ST-7 formats         *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.15:  6-APR-96 Overwholing                                     *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.14:  1-APR-96 Added gray strips for FITS files + 12 lines     *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.13: 30-MAR-96 First release                                   *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.03: 30-MAR-96 Bug fixes and corrections                       *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.02: 20-MAR-96 Continued with TIFF-to-FITS                     *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.01: 10-FEB-96 Copied and extracted from CCDCFUNC.INC          *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.00: 25-JUN-94 Extracted from CCDCPROC.INC                     *
'* ------------------------------------------------------------------------ *
'* COMMENTS:                                                                *
'* ======================================================================== *
'* AUTHOR: Hans-Ruedi H. Wernli, Pletschgasse, CH-3952 Susten, Switzerland. *
'****************************************************************************
'
'
CCDFFuncV% = 116
'
'
' FUNCTION  returns true if filesize A$ = A& else false
'----------------------------------------------------------------------------
'
function AbsSize%(A$,A&)
  local B&
  open A$ for input as # %UnitAUX
  B& = lof(%UnitAUX)
  close %UnitAUX
  if A& = B& then AbsSize% = %True else AbsSize% = %False
end function
'
'
' FUNCTION  returns true if filesize A$ is correct for Byte-files, else false
'----------------------------------------------------------------------------
'
function BytSize%(A$)
  local D$,Z??
  open A$ for binary as # %UnitAUX
  if lof(%UnitAUX) > 5 then
    get$ # %UnitAUX, 5,D$
    close %UnitAUX
    Z?? = asc(mid$(D$,4,1)) * 256 + asc(mid$(D$,3,1))
    if Z?? = 31680 then BytSize% = %True else BytSize% = %False
  else
    BytSize% = %False
  end if
end function
'
'
' FUNCTION  returns true if filesize A$ is correct for PIC files, else false
'----------------------------------------------------------------------------
'
function PicSize%(A$)
  local Z&,D$,A%,S%,L&
  S% = 3
  open A$ for binary as # %UnitAUX
  Z& = lof(%UnitAUX)
  if Z& > 3 then
    get$ # %UnitAUX, 4, D$
    A% = StrInt(S%,D$)
    if Z& > A% then
      seek %UnitAUX, 0
      get$ # %UnitAUX, A%, D$
      S% = 69
      L& = StrInt%(S%,D$) * StrInt%(S%,D$) * (StrInt%(S%,D$)\8) + A%
      if L& = Z& then PicSize% = %True
    else
      PicSize% = %False
    end if
  else
    PicSize% = %False
  end if
  close %UnitAUX
end function
'
'
' FUNCTION  returns true if filesize in A$ is correct for ST6 files
'   Since ST6 files could be compressed, exact data size cannot be
'   determined. Therefore, the file must be longer than the header
'   (2048 bytes) and the first line must read ST-6 [Compressed] Image.
'   This function is also called for ST-7 files (if T%=3)
'----------------------------------------------------------------------------
'
function ST6Size%(A$,T%)
  local S$,Z$
  if T% = 2 then Z$ = "ST-6 " else Z$ = "ST-7 "
  open A$ for input as # %UnitAUX
  if lof(%UnitAUX) > 2048 then
    line input# %UnitAUX, D$
    if left$(D$,5) = Z$ then
      if right$(D$,7) = " Image" + chr$(10) then ST6Size% = %True
    else
      ST6Size% = %False
    end if
  else
    ST6Size% = %False
  end if
  close %UnitAUX
end function
'
'
' FUNCTION  returns true if filesize is a multiple of 2880 bytes, at least
'   once.
'----------------------------------------------------------------------------
'
function FitSize%(A$)
  local Z&,A%,B%
  open A$ for input as # %UnitAUX
  Z& = lof(%UnitAUX)
  close %UnitAUX
  A% = Z& \ 2880
  B% = Z& mod 2880
  if (B% = 0) and (A% > 0) then FitSize% = %True else FitSize% = %False
end function
'
'
' FUNCTION returns integer from string
'   S%=Start in string D$, D$ = raw head. S% = S%+2 on exit
'----------------------------------------------------------------------------
'
function StrInt%(S%,D$)
  local H%,L%
  L% = asc(mid$(D$,S%,1))
  incr S%
  H% = asc(mid$(D$,S%,1))
  incr S%
  StrInt% = H% * 256 + L%
end function
'
'
' FUNCTION  writes FITS Head into file A$ and returns file handle
'----------------------------------------------------------------------------
'
function WrFitH%(A$,Card$(),FOpen%)
  local H%,N%
  open A$ for binary as # %UnitFIT
  H% = fileattr(%UnitFIT,2)
  for N% = 1 to 36
    put$ # %UnitFIT, Card$(N%)
  next N%
  FOpen% = %True
  WrFitH% = H%
end function
'
'
' FUNCTION  returns from ST4 line 166 in R$ date as dd-mon-yy
'----------------------------------------------------------------------------
'
function ST4Date$(R$)
  local N%,M%,M$
ST4MoList:
  data JAN,FEB,MAR,APR,MAY,JUN,JUL,AUG,SEP,OCT,NOV,DEC
  M% = val(mid$(R$,16,2))
  restore ST4MoList
  for N% = 1 to M%
    read M$
  next N%
  ST4Date$ = mid$(R$,19,2) + "-" + M$ + "-" + mid$(R$,22,2)
end function
'
'
' FUNCTION  returns from ST4 line 166 in R$ time as hh:mm:ss
'----------------------------------------------------------------------------
'
function ST4Time$(R$)
  ST4Time$ = mid$(R$,28,8)
end function
'
'
' FUNCTION  returns from ST4 line 166 in R$ integration time in seconds
'----------------------------------------------------------------------------
'
function ST4Integ$(R$)
  local A!,A$
  A! = val(mid$(R$,80,10)) / 100
  A$ = str$(A!)
  ST4Integ$ = ltrim$(rtrim$(A$)) + " s"
end function
'
'
' FUNCTION  returns from ST4 line 166 in R$ focal length in mm
'----------------------------------------------------------------------------
'
function ST4FocLen$(R$)
  local A!,A$
  A! = val(mid$(R$,90,10)) * 25.4
  A$ = str$(A!)
  ST4FocLen$ = ltrim$(rtrim$(A$)) + " mm"
end function
'
'
' FUNCTION  returns from ST4 line 166 in R$ apperture diameter in mm
'----------------------------------------------------------------------------
'
function ST4Apert$(R$)
  local A!,B!,A$
  A! = val(mid$(R$,100,10))
  B! = sqr(A!) * 25.4
  A$ = str$(B!)
  ST4Apert$ = ltrim$(rtrim$(A$)) + " mm"
end function
'
'
' FUNCTION  returns from ST4 line 166 in R$ calibration factor
'----------------------------------------------------------------------------
'
function ST4Calib$(R$)
  local A!,A$
  A! = val(mid$(R$,110,10))
  A$ = str$(A!)
  ST4Calib$ = ltrim$(rtrim$(A$))
end function
'
'
' FUNCTION  returns true if ST6 or ST7 file is compressed, else false
'----------------------------------------------------------------------------
'
function ST6Comp%(R$)
  local Z%,A$
  Z% = instr(R$,chr$(10))
  A$ = lcase$(left$(R$,Z%))
  if instr(A$,"compressed") then ST6Comp% = %True else ST6Comp% = %False
end function
'
'
' FUNCTION  returns string with value as string from R$ mask M$ for ST6
'----------------------------------------------------------------------------
'
function FindVal$(R$,M$)
  local Z%,A%,A$,D$
  Z% = instr(1,R$,M$)
  if Z% > 1 then
    A% = instr(Z%,R$,chr$(10))
    if A% > 0 then
      A$ = mid$(R$,Z%,A%-Z%)
      D$ = remove$(A$,M$)
      FindVal$ = remove$(D$," ")
    else
      FindVal$ = "?"
    end if
  else
    FindVal$ = "?"
  end if
end function
'
'
' FUNCTION  returns date when ST6 or ST7 image was taken as dd-mon-yy
'----------------------------------------------------------------------------
'
function ST6Date$(R$)
  local T%,M%,Y%,N%,D$,M$
ST6MoList:
  data JAN,FEB,MAR,APR,MAY,JUN,JUL,AUG,SEP,OCT,NOV,DEC
  D$ = FindVal$(R$,"Date =")
  if len(D$) = 8 then
    T% = val(mid$(D$,3,2))
    M% = val(left$(D$,2))
    Y% = val(right$(D$,2))
    if (T% = 0) or (M% = 0) or (Y% = 0) then
      ST6Date$ = "dd-mmm-yy"
    else
      if (M% < 1) or  (M% > 12) then swap M%,T%
      if (M% > 0) and (M% < 13) then
        restore ST6MoList
        for N% = 1 to M%
          read M$
        next N%
        D$ = remove$(str$(T%)," ")
        ST6Date$ = D$ + "-" + M$ + "-" + remove$(str$(Y%)," ")
      else
        ST6Date$ = "dd-mmm-yy"
      end if
    end if
  else
    ST6Date$ = "dd-mmm-yy"
  end if
end function
'
'
' FUNCTION  returns time when ST6 or ST7 image was taken as hh:mm:ss
'----------------------------------------------------------------------------
'
function ST6Time$(R$)
  local D$
  D$ = FindVal$(R$,"Time =")
  if len(D$) = 8 then ST6Time$ = D$ else ST6Time$ = "hh:mm:ss"
end function
'
'
' FUNCTION  returns focal length of ST6 or ST7 image in mm
'----------------------------------------------------------------------------
'
function ST6FocLen$(R$)
  local A!,D$
  D$ = FindVal$(R$,"Focal_length =")
  if D$ = "?" then
    ST6FocLen$ = D$ + " mm"
  else
    A! = val(D$) * 25.4
    D$ = using$("#####.###",A!)
    ST6FocLen$ = remove$(D$," ") + " mm"
  end if
end function
'
'
' FUNCTION  returns apperture diameter of ST6 or ST7 image in mm
'----------------------------------------------------------------------------
'
function ST6Apert$(R$)
  local A!,B!,D$
  D$ = FindVal$(R$,"Aperture =")
  if D$ = "?" then
    ST6Apert$ = D$ + " mm"
  else
    B! = val(D$)
    A! = sqr(B!) * 25.4
    D$ = using$("#####.###",A!)
    ST6Apert$ = remove$(D$," ") + " mm"
  end if
end function
'
'
' FUNCTION  returns Naxis1 ST6 or ST7 image
'----------------------------------------------------------------------------
'
function ST6N1%(R$)
  local D$
  D$ = FindVal$(R$,"Width =")
  ST6N1% = val(D$)
end function
'
'
' FUNCTION  returns Naxis2 ST6 or ST7 image
'----------------------------------------------------------------------------
'
function ST6N2%(R$)
  local D$
  D$ = FindVal$(R$,"Height =")
  ST6N2% = val(D$)
end function
'
'
' FUNCTION returns start of image data in PIC-file. File assumed to be open
'----------------------------------------------------------------------------
'
function PicStart%(D$)
  local A%
  A% = 3
  get$ # %UnitAUX, 128, D$
  PicStart% = StrInt%(A%,D$)
end function
'
'
' FUNCTION returns WORD from a FITS Card
'----------------------------------------------------------------------------
'
function CardVal??(Z$)
  local A$,B$
  A$ = left$(Z$,30)
  B$ = right$(A$,21)
  A$ = remove$(B$," ")
  CardVal?? = val(A$)
end function
'
'
' FUNCTION returns index for card with a particular searchtext.
'   If no match found, returns index 0
'----------------------------------------------------------------------------
'
function CardNdx%(Card$(),Z$)
  local N%
  for N% = 1 to 36
    if left$(Card$(N%),8) = Z$ then exit for
  next N%
  if N% = 37 then CardNdx% = 0 else CardNdx% = N%
end function
'
'
' FUNCTION returns ASCIIZ from a comment or text card
'----------------------------------------------------------------------------
'
function CardAscZ$(Z$)
  local A$
  A$ = right$(Z$,71)
  CardAscZ$ = ltrim$(rtrim$(A$)) + chr$(0)
end function
'
'
' FUNCTION returns date and time as asciiz for Tag 306
'----------------------------------------------------------------------------
'
function DTAscZ$()
  local A$,B$
  A$ = date$
  B$ = right$(A$,4) + ":" + left$(A$,2) + ":" + mid$(A$,4,2) + " "
  DTAscZ$ = B$ + time$ + chr$(0)
end function
'
'
' FUNCTION returns true if it generated gray scale for inclusion at the
'   end of the FITS image, else false
'---------------------------------------------------------------------------
'
function GrayScale%(CCA%(),CCD%(2),BP%,N1%,Hi??,Lo??,E%)
  local D??,H??,L??,N!,P%
  if N1% < 2641 then
    call FillWC(CCD%(1,1),0%,%LMN)
    call FillWC(CCA%(1),  0%,%LMN)
    H?? = Hi??
    L?? = Lo??
    if H?? > 32767 then H?? = 32767
    if L?? > 32767 then L?? = 32767
    D?? = H?? - L??
    if D?? < 2 then D?? = 2
    S! = D?? / N1%
    P% = 1
    for N! = L?? to H?? step S!
      CCA%(P%) = int(N!)
      incr P%
    next N!
    call Rplcte(CCD%(1,1),CCA%(1),N1%)
    E% = N1% * 12
    if BP% = 8 then
      call CpyW2W(CCA%(1),CCD%(1,1),E%)
      call CpyW2B(CCD%(1,1),CCA%(1),E%)
    else
      call MotInt(CCD%(1,1),E%)
    end if
    GrayScale% = %True
  else
    GrayScale% = %False
  end if
end function
'
'
' FUNCTION returns false if process was successfull, else SLText-Nr.
'   AUXUnit is read, Hi and Lo searched for, then FITUnit written
'----------------------------------------------------------------------------
'
function RdAWrF%(CCA%(),Handla%,Handle%,BL%,BP%,Hi??,Lo??,MF%,XF%)
  local H%,Z%,SF%,L??,H??
  Z% = RdFit%(Handla%,BL%,CCA%(1))
  if isfalse Z% then
    if BP% = 16 then
      H% = BL% \ 2
      if istrue MF% then call MotInt(CCA%(1),H%)
      if istrue XF% then call ShrMWA(CCA%(1),2?,H%)
      call AndMWA(CCA%(1),32767??,H%)
      L?? = MinWrd??(CCA%(1),H%)
      H?? = MaxWrd??(CCA%(1),H%)
      call MotInt(CCA%(1),H%)
      SF% = 1
    else
      H% = BL%
      L?? = MinByt??(CCA%(1),H%)
      H?? = MaxByt??(CCA%(1),H%)
      SF% = 0
    end if
    if L?? < Lo?? then Lo?? = L??
    if H?? > Hi?? then Hi?? = H??
    Z% = WrFits%(H%,CCA%(1),Handle%,SF%)
    if isfalse Z% then
      RdAWrF% = %False
    else
      RdAWrF% = 74
    end if
  else
    RdAWrF% = 72
  end if
end function
'
'
' FUNCTIONS FOR TIFF-HEAD ANALYSIS
'============================================================================
'
' FUNCTION  returns from 2-byte string integer value.
'   The string may contain the value in Intel- or Motorola format.
'----------------------------------------------------------------------------
'
function ShortInt??(R$,M%)
  local A%,B%
  A% = asc(left$(R$,1))
  B% = asc(right$(R$,1))
  if istrue M% then swap A%,B%
  ShortInt?? = B% * 256 + A%
end function
'
'
' FUNCTION  returns from 4-byte string long integer value.
'   The string may contain the value in Intel- or Motorola format.
'----------------------------------------------------------------------------
'
function LongInt???(R$,M%)
  local A%,B%,C%,D%,L&&
  A% = asc(left$(R$,1))
  B% = asc(mid$(R$,2,1))
  C% = asc(mid$(R$,3,1))
  D% = asc(right$(R$,1))
  if istrue M% then
    swap A%,D%
    swap B%,C%
  end if
  LongInt??? = D% * 16777216& + C% * 65536?? + B% * 256% + A%
end function
'
'
' FUNCTION  returns true for Motorola, false for Intel and 5 for none
'----------------------------------------------------------------------------
'
function IIMM%(R$)
  local A$
  A$ = left$(R$,2)
  if A$ = "MM" then
    IIMM% = %True
  else
    if A$ = "II" then
      IIMM% = %False
    else
      IIMM% = 5
    end if
  end if
end function
'
'
' FUNCTION  returns TIFF-version, usually 42.
'   42 is the anser to the Life, the Universe and everything.
'----------------------------------------------------------------------------
'
function TiffV%(R$,M%)
  local A??,Z$
  Z$ = mid$(R$,3,2)
  A?? = ShortInt??(Z$,M%)
  if A?? > 32767 then A?? = A?? - 32767
  TiffV% = A??
end function
'
'
' FUNCTION  returns the address of the Image File Directory
'----------------------------------------------------------------------------
'
function TiffIFD&(R$,M%)
  local A%,Z$
  Z$ = right$(R$,4)
  TiffIFD& = LongInt???(Z$,M%)
end function
'
'
' FUNCTION  returns from the IFD the number of Tags that follow
'----------------------------------------------------------------------------
'
function Tagx%(R$,M%,IFD&)
  local T%,L&,Z$
  L& = lof(%UnitAUX)
  if L& < IFD& + 2 then
    Tagx% = 0
  else
    seek %UnitAUX, IFD&
    get$ # %UnitAUX, 2, Z$
    T% = ShortInt??(Z$,M%)
    if L& < T% * 12 + 8 then T% = 0
    Tagx% = T%
  end if
end function
'
'
' FUNCTION returns a 70 character$ from tags 305, 306, 315 and 316
'----------------------------------------------------------------------------
'
function TiffTxt$(F$,A???,B???,F%)
  local D$,R$,A%
  A% = A???
  if A% = 1 then A% = 256
  open F$ for binary as # %UnitAUX
  seek# %UnitAUX, B???
  get$# %UnitAUX, A%, R$
  close %UnitAUX
  A% = instr(R$,chr$(0))
  if A% > 0 then R$ = left$(R$,A%)
  D$ = remove$(R$,chr$(0))
  if istrue F% then R$ = "'" + D$ + "'" else R$ = D$
  TiffTxt$ = left$(R$ + space$(70),70)
end function
'
'
' FUNCTION returns a pointer value from the IFD
'  J is index, M the Motorola-Flag, T the datatype and S the offset
'----------------------------------------------------------------------------
'
function TiffPtr???(J%,M%,T???,S???)
  local A%,B%,A???,D$
  if T??? = 4 then B% = 4 else B% = 2
  A??? = (J%-1) * B% + S???
  seek# %UnitAUX, A???
  get$# %UnitAUX, 4, D$
  TiffPtr??? = LongInt???(D$,M%)
end function
'
'
' FUNCTION returns from WORD (??) a 2-byte string in the Intel format
'----------------------------------------------------------------------------
'
function Single$(A??)
  local A$
  A$ = chr$(0,0)
  call SgWrdS(A??,A$)
  Single$ = A$
end function
'
'
' FUNCTION returns from DWORD (???) a 4-byte string in the Intel format
'----------------------------------------------------------------------------
'
function Double$(A???)
  local A$
  A$ = chr$(0,0,0,0)
  call DbWrdS(A???,A$)
  Double$ = A$
end function
'
'
' FUNCTION returns 22 formatted Tags X=1 for NAXIS=2 X=3 for NAXIS=3
'----------------------------------------------------------------------------
'
function TiffH$(N1??,N2??,X%,CCD%(2),SN???,SL???,SB???,RL???,RB???)
  local F%,NX??,V???,W???,A$,B$,C$,D$,E$,F$,G$,H$
  NX?? = N2?? * X%
  if (SL??? = NX??) or (RL??? = NX??) then F% = %True else F% = %False
  A$ = Single$(254??) + Single$(4??) + Double$(1???) + Double$(0???)
  V??? = N1??
  B$ = Single$(256??) + Single$(3??) + Double$(1???) + Double$(V???)
  V??? = N2??
  C$ = Single$(257??) + Single$(3??) + Double$(1???) + Double$(V???)
  D$ = Single$(258??) + Single$(3??) + Double$(1???) + Double$(8???)
  E$ = Single$(259??) + Single$(3??) + Double$(1???) + Double$(1???)
  F$ = Single$(262??) + Single$(3??) + Double$(1???) + Double$(1???)
  H$ = A$ + B$ + C$ + D$ + E$ + F$
  V??? = CCD%(1,1)
  W??? = CCD%(2,1)
  A$ = Single$(269??) + Single$(2??) + Double$(W???) + Double$(V???)
  V??? = CCD%(1,2)
  W??? = CCD%(2,2)
  B$ = Single$(270??) + Single$(2??) + Double$(W???) + Double$(V???)
  V??? = CCD%(1,3)
  W??? = CCD%(2,3)
  C$ = Single$(271??) + Single$(2??) + Double$(W???) + Double$(V???)
  V??? = CCD%(1,4)
  W??? = CCD%(2,4)
  D$ = Single$(272??) + Single$(2??) + Double$(W???) + Double$(V???)
  V??? = CCD%(2,9)
  if istrue F% then
    W??? = 1???
  else
    if RL??? > 0 then incr SN???
    W??? = SN???
  end if
  E$ = Single$(273??) + Single$(4??) + Double$(W???) + Double$(V???)
  G$ = H$ + A$ + B$ + C$ + D$ + E$
  A$ = Single$(274??) + Single$(3??) + Double$(1???) + Double$(1???)
  B$ = Single$(277??) + Single$(3??) + Double$(1???) + Double$(1???)
  if SL??? = 0 then V??? = RL??? else V??? = SL???
  C$ = Single$(278??) + Single$(4??) + Double$(1???) + Double$(V???)
  if istrue F% then
    V??? = N1?? * NX??
    W??? = 1???
  else
    V??? = SN??? * 4 + CCD%(2,9)
    W??? = SN???
  end if
  D$ = Single$(279??) + Single$(4??) + Double$(W???) + Double$(V???)
  V??? = CCD%(1,9)
  E$ = Single$(282??) + Single$(5??) + Double$(1???) + Double$(V???)
  F$ = Single$(283??) + Single$(5??) + Double$(1???) + Double$(V???)
  H$ = G$ + A$ + B$ + C$ + D$ + E$ + F$
  if X% = 1 then V??? = 1 else V??? = 2
  A$ = Single$(284??) + Single$(3??) + Double$(1???) + Double$(V???)
  V??? = CCD%(1,5)
  W??? = CCD%(2,5)
  B$ = Single$(305??) + Single$(2??) + Double$(W???) + Double$(V???)
  V??? = CCD%(1,6)
  W??? = CCD%(2,6)
  C$ = Single$(306??) + Single$(2??) + Double$(W???) + Double$(V???)
  V??? = CCD%(1,7)
  W??? = CCD%(2,7)
  D$ = Single$(315??) + Single$(2??) + Double$(W???) + Double$(V???)
  V??? = CCD%(1,8)
  W??? = CCD%(2,8)
  E$ = Single$(316??) + Single$(2??) + Double$(W???) + Double$(V???)
  F$ = Single$(0???) + Double$(300??) + Double$(4???)
  G$ = H$ + A$ + B$ + C$ + D$ + E$ + F$
  A$ = ""
  if isfalse F% then
    V??? = (SN??? - 1) * 8 + CCD%(2,9)
    W??? = SB???
    for I% = 1 to SN???
      A$ = A$ + Double$(V???)
      V??? = V??? + W???
    next I%
    for I% = 2 to SN???
      A$ = A$ + Double$(W???)
    next I%
    if RL??? > 0 then W??? = RB???
    A$ = A$ + Double$(W???)
  end if
  TiffH$ = G$ + A$
end function
'
'
' FUNCTION  returns RowsPerStrip-278 (SL), number of full strips (SN),
'   number of rest strips (RN=0 or 1), StripByteCount-279 (SB) and
'   StripByteCount for rest strip (RB). N1 is Naxis1, N2 Naxis2 and
'   X is 1 for monochrome images and 3 for RGB images.
'----------------------------------------------------------------------------
'
function Stripses???(X%,N1??,N2??,SL???,SB???,RL???,RB???)
  local NX??, SN???
  NX?? = N2?? * X%
  SL??? = 65536 \ N1??
  SB??? = N1?? * SL???
  SN??? = NX?? \ SL???
  RL??? = NX?? - (SN??? * SL???)
  RB??? = RL??? * N1??
  if SN??? = 0 then
    SL??? = 0
    SB??? = 0
  end if
  Stripses??? = SN???
end function
'
'
'===============================-<960211>-============-<HoroSoft>-===========
