'970413 1026
'****************************************************************************
'* PROGRAM NAME: CCDFITS .BAS   DATE: 10. 2.1996   INCLUDE IN: is main      *
'* LANGUAGE: Power BASIC 3.20  (Robert S.Zale)                              *
'* PURPOSE: Converts different CCD picture formats into FITS.               *
'* INCLUDE FILES: CCDFHEAD.INC  Header file with declarations               *
'*                ERROR3  .INC  Fatal error handler                         *
'*                IOTEXT  .INC  Set of Text and File functions              *
'*                CCDFPROC.INC  Procedures                                  *
'*                CCDFFUNC.INC  Functions                                   *
'*                CCDFDAT .INC  English and German Texts                    *
'*                CCDFSUB1.INC  Input/Output subroutines - converters       *
'*                CCDFSUB2.INC  Sub-Subroutines and commons                 *
'* OBJECT FILES:  PBSTACK. OBJ  Power BASIC 2 & 3 Stack handler for objects *
'*                ARRAYEXT.PBL  Advanced array manipulation functions)      *
'*                CLRSCRN .PBL  Hoopy frood clear screen routines           *
'*                RLCEGEN .OBJ  Run Length Count Encoding Generator         *
'*                CCDFF   .OBJ  procedures                                  *
'* UNIT FILES:    PB3GPUSE.PBU  General Purpose Functions and Procedures    *
'* INPUT FILES:   filename.LNX  Lynxx image                                 *
'*                filename.ST4  ST-4 image                                  *
'*                filename.ST6  ST-6 image                                  *
'*                filename.ST7  ST-7 image                                  *
'*                filename.SXP  Starlight X-Press                           *
'*                filename.PIC  QMiPS format                                *
'*                filename.PIX  QuickPix format                             *
'*                filename.BYT  CCDUTIL image format                        *
'*                filename.RAW  unknown raster image data                   *
'*                filename.TIF  Tagged Image File Format                    *
'*                filename.FIT  to edit FITS-Header, remove graybar & TIFF  *
'* OUTPUT FILES:  filename.FIT  FITS format                                 *
'*                filename.TIF  Tagged Image File Format                    *
'*                filename.ERR  If TIFF raster data is not supported        *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.18 13-APR-93 Added Starlight Xpress and ST-7 formats          *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.17  6-APR-96 Becomes mature                                   *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.16 30-MAR-96 First release                                    *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.06 30-MAR-96 Added Submenu for FITS converters                *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.05 22-MAR-96 Added TIFF and FITS converters                   *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.04 18-MAR-96 Added Header file philosophy as a test           *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.03 11-MAR-96 Added RAW-format, inspired by IMDISP             *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.02 10-FEB-96 Started to make it for all formats               *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.01  4-FEB-96 Found out that data starts at end-line           *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.00  1-FEB-96 Start of labour for MiPS PIC files               *
'* ------------------------------------------------------------------------ *
'* COMMENTS: PIC-head not fully analyzed, but sufficiently                  *
'* ------------------------------------------------------------------------ *
'* NOTE: For file formats, see listing of CCDUTIL.                          *
'* ------------------------------------------------------------------------ *
'* TESTING: FITS: IMDISP V.7.7 ImageDisplay by Jet Propulsion Laboratory    *
'*                Problem: cannot read files with more than 2 axis.         *
'*                WINMiPS V.1.5c by Christian Buil                          *
'*                Problem: has problems while loading. Reason unknown!      *
'*                Graphics Workshop for Windows GWSWIN V.1.1n               *
'*                Problem: reads only lower bytes in 16bit images           *
'*          TIFF: PaintShopPro V.3.0, JASC, Inc.:                          *
'*                Graphic Workshop for Windows V.1.1j, Alchemy MW.:        *
'*                Graphic Workshop V.6.1w, Alchemy Mindworks, Inc.:        *
'*                PhotoTouch V.2.1, Logitech SA, Switzerland:              *
'*                iPhoto Plus V.1.01, U-Lead Sys., Inc.:                   *
'*                Designer Plus OLE V.3.1, Micrografx; Import:             *
'*                Designer V.4.1a, Micrografx; Import:                     *
'*                Picture Publisher V.5.0a, Micrografx:                    *
'*                HiJaak Pro V.3.0, Inset Systems, Inc.:                   *
'*                HiJaak Paint V.3.0, Media Cybernetics, Inc.:             *
'*                HiJaak Draw V.1.0a, Gold Disk Inc.: Error on Import       *
'*                Norton Commander V.5.0, Symantec Inc.:                   *
'*                Claris Works 1.0D V.1, Claris Corporation:               *
'*                Claris Works 3.0  V.2, Claris Corporation:               *
'*                CCDPRINT V.2.71, HoroSoft, Switzerland: output           *
'* ======================================================================== *
'* AUTHOR: Hans-Ruedi H. Wernli, Pletschgasse, CH-3952 Susten, Switzerland. *
'****************************************************************************
'
'
' FILES TO BE INCLUDED
'-----------------------------------------------------------------------------
'
$include "CCDFHEAD.INC"		'header with directives and declarations
$include "ERROR3.INC"		'omni error handler
$include "IOTEXT.INC"		'omni input/output procedures
$include "CCDFPROC.INC"		'dedicated procedures
$include "CCDFFUNC.INC"		'dedicated functions
$include "CCDFSUB2.INC"		'general purpose subroutines
$include "CCDFSUB1.INC"		'menu execution
$include "CCDFDAT.INC"		'texts
$segment
'
'
' FILES TO BE LINKED IN
'-----------------------------------------------------------------------------
'
$link "PBSTACK.OBJ"		'omni stack handler for external objects
$link "ARRAYEXT.PBL"		'library of array manipulation procedures
$link "CCDFF.OBJ"		'dedicated objects
$link "RLCEGEN.OBJ"		'RLCE codec
$link "PB3GPUSE.PBU"		'omni procedures
$link "CLRSCRN.PBL"		'clear screen routines
'
'
' PROGRAM SETUP
'============================================================================
'
' LANGUAGE SELECTION
'----------------------------------------------------------------------------
'
English% = ComLinLan%(command$,"CCDTOOLS: "+VN$)
if English% then
  Idiom%(0) = %True		'English
  Idiom%(1) = 0			'SL offset
  Idiom%(2) = 0			'ML offset
  Version$ = "CCD Image File Format Converter " + VN$
else
  Idiom%(0) = %False		'Deutsch
  Idiom%(1) = 100		'SL offset
  Idiom%(2) = 20		'ML offset
  Version$ = "CCD Bild Datei Format Konverter " + VN$
end if
'
'
' SET DEFAULTS
'-----------------------------------------------------------------------------
'
Virgin% = %True
InpPath$ = curdir$
OutPath$ = curdir$
'
'
' CHECK FREE MEMORY
'   FRE returns free memory on start up. Program needs at least 64K more,
'   for security reasons, an additional 32K are reserved, that makes 100K
'   Tested with "SHELL MAPMEM" at start and end of program.
'----------------------------------------------------------------------------
'
Z% = fix(fre(-1)/1024)
Fatal$ = SLText$(57%) + str$(100-Z%) + "Kb"
if Z% < 100 then goto NoJob			'Needs about 504Kb
'
'
' START OF PROGRAM
'============================================================================
'
'------------------------------ Log on --------------------------------------
'
print LogOn$(Version$,%Cyan,%Black)
print Describe$
'
'
'=============================-<Start of other Conversion Loop>-=============
do
  for N% = 1 to 16
    BatchI$(N%) = Nill$
    BatchO$(N%) = Nill$
    Batch%(N%) = %False
  next N%
'
' Log On and input option gathering: 1...9
'
  print LogOn$(Version$,%Cyan,%Black);
  print Describe$
  Y% = csrlin
  locate Y%,1
  color %LCyan,%Black
  print Center$(SLText$(5%),80)
  color %Cyan,%Black
  call MLText(2%,15%,"tf")
'
'-|-------------------------------- Start of Input Option Loop --------------
  do
    InpOpt% = Prompt%(20%,Q$)
    if InpOpt% > -1 and InpOpt% < 10 then exit loop
    print Outch$(Q$)
  loop
  if InpOpt% = 0 then exit loop
'
'.|................................ End of Input Option Loop ................
'
'-- Prepare file extensions
'
  if InpOpt% = %STX then
    print LogOn$(Version$,%Cyan,%Black);
    print Describe$
    Y% = csrlin
    locate Y%,1
    color %LCyan,%Black
    print Center$(SLText$(5%),80)
    color %Cyan,%Black
    call MLText(4%,28%,"tf")
    do
      SBIG% = Prompt%(33%,Q$)
      if SBIG% > -1 and SBIG% < 4 then exit loop
      print Outch$(Q$)
    loop
    select case SBIG%
      case 0
        exit loop
      case 1
        IOExt$(InpOpt%) = "ST4"
      case 2
        IOExt$(InpOpt%) = "ST6"
      case 3
        IOExt$(InpOpt%) = "ST7"
    end select
  end if
  InpExt$ = IOExt$(InpOpt%)
  if InpOpt% = %FTI then OutExt$ = IOExt$(%TIF) else OutExt$ = IOExt$(%FIT)
'
'
'-- Get Output path
'
  print LogOn$(Version$,%Brown,%Black);
  print Describe$
  I% = %False
  gosub GetDirPath
  OutPath$ = UsePath$
  if Virgin% then InpPath$ = UsePath$	'-- only the first time
'
'-|------------------------- Start of Input Path Loop ----------------------
  do
    print LogOn$(Version$,%Green,%Black);
    I% = %True
    gosub GetDirPath			'-- Get input path
    InpPath$ = UsePath$
    A% = ListFiles%(InpPath$,InpExt$,English%)
    if A% then exit loop
    beep
    Fatal$ = SLText$(3%)
    if not Redo%(SLText$(39%)) then goto NoJob
  loop
'.|............................... End of Input Path Loop ...................
'
  Y? = csrlin
  if Y? > 21 then
    for N% = 1 to 7
      print
    next N%
  end if
  locate 21,1
'
'-- Input file gathering
'
  color %Black,%Cyan
  print Center$(SLText$(2%),80)
  color %Cyan,%Black
  Y% = csrlin
  for N% = 1 to 16
    locate Y%,1
    print space$(80);
    locate Y%,1
    A$ = IOFile$(English%,%False,SLText$(36%))
    if instr(A$,"END") then exit for
    BatchF$(N%) = StripExt$(A$) + "." + InpExt$
    BatchI$(N%) = InpPath$ + BatchF$(N%)
    BatchO$(N%) = OutPath$ + StripExt$(A$) + "." + OutExt$
  next N%
  NoFiles% = N% - 1
'
'-- Filespec and filesize validification, construction of output filespecs
'
  print Logon$(Version$,%Cyan,%Blue);
  color %LCyan,%Black
  print Center$(SLText$(4%)+Space$(6)+"("+SLText$(16%)+")",80)
  for N% = 1 to NoFiles%
    color %White,%Black
    print "[";
    print using "##";N%;
    print "] ";
    color %Green,%Black
    print BatchI$(N%);" ";
    if not dir$(BatchI$(N%)) = "" then
      select case InpOpt%
        case %LNX
          Batch%(N%) = AbsSize%(BatchI$(N%),LNXSize&)
          call IssueOk(Batch%(N%))
        case %STX
          select case SBIG%
            case 1
              Batch%(N%) = AbsSize%(BatchI$(N%),ST4Size&)
              call IssueOk(Batch%(N%))
            case 2, 3
              Batch%(N%) = ST6Size%(BatchI$(N%),SBIG%)
              call IssueOk(Batch%(N%))
          end select
        case %SXP
          Batch%(N%) = AbsSize%(BatchI$(N%),SXPSize&)
          call IssueOk(Batch%(N%))
        case %PIC
          Batch%(N%) = PicSize%(BatchI$(N%))
          call IssueOk(Batch%(N%))
        case %PIX
          Batch%(N%) = AbsSize%(BatchI$(N%),PIXSize&)
          call IssueOk(Batch%(N%))
        case %BYT
          Batch%(N%) = BytSize%(BatchI$(N%))
          call IssueOK(Batch%(N%))
        case %RAW
          Batch%(N%) = FileHere%(BatchI$(N%))
          call IssueOk(Batch%(N%))
        case %TIF
          Batch%(N%) = FileHere%(BatchI$(N%))
          call IssueOk(Batch%(N%))
        case %FTI			'FITS-->TIFF & FITS-Head
            Batch%(N%) = FitSize%(BatchI$(N%))
            call IssueOk(Batch%(N%))
      end select
    else
      color %LRed,%Black
      print SLText$(19%)
      Batch%(N%) = %False
    end if
  next N%
'
'-- Moving filespecs such that invalid ones are skipped
'
  Cnt% = 0
  for N% = 1 to NoFiles%
    if Batch%(N%) then
      incr Cnt%
      BatchI$(Cnt%) = BatchI$(N%)
      BatchO$(Cnt%) = BatchO$(N%)
    end if
  next N%
  NoFiles% = Cnt%
  Q$ = KeyIn$
'
' Verify to overwrite or skip existing files, if any - but only, if not
' FIT to FIT: edit Fits-Head or Remove Graybar
'
  print LogOn$(Version$,%White,%Red)
  color %Cyan,%Black
  print Center$(SLText$(11%),80)
  for N% = 1 to NoFiles%
    C$ = right$(BatchI$(N%),3) + right$(BatchO$(N%),3)
    if isfalse C$ = "FITFIT" then
      if not dir$(BatchO$(N%)) = "" then
        color %Green,%Black
        print BatchO$(N%);" ";
        color %LRed,%Black
        print SLText$(8%)
        if Confirm%(SLText$(9%)) then
          kill BatchO$(N%)
        else
          Batch%(N%) = %False
        end if
      end if
    end if
  next N%
  color %LWhite,%Black
'
' Now we can start doing the job. We select the input menu and for each one
'
  print LogOn$(Version$,%White,%Blue)
  color %Cyan,%Black
  print SLText$(17%);
  Y? = csrlin
  X? = pos(Y?)
  for N% = 1 to NoFiles%
    locate Y?,X?
    color %LCyan,%Black
    print using "###";NoFiles%-N%+1
    color %Cyan,%Black
    print
    ST4Info$ = chr$(190) + space$(190) + chr$(0)
    if Batch%(N%) then
      call FillWC(CCD%(1,1),0%,%LMN)
      call FillWC(CCQ%(1,1),0%,%LMN)
      on InpOpt% gosub ILNX,ISTX,ISXP,IPIC,IPIX,IBYT,IRAW,ITIF,FITSub
    end if
    for F? = Y?+1 to 20
      locate F?,1
      call ClrLine
    next F?
  next N%
  locate Y?,X?
  color %LCyan,%Black
  print using "###";NoFiles%-N%+1
  color %Cyan,%Black
'
' Redo all with different input and output menus
'
  print
  Virgin% = %False
  print LogOn$(Version$,%White,%Blue)
  if not Confirm%(SLText$(7%)) then exit loop
loop
'
'::::::::::::::::::::::::::::::: End of other Conversion Loop :::::::::::::::
'
' That's it - we're going to log off
'
LogOff:
'
print LogOn$(Version$,%White,%Blue);
print Describe$
color %LWhite,%Green
A$ = VN$ + SLText$(30%)
print Center$(A$,80)
color %LWhite,%Red
A$ = VN$ + "; " + GPCR$
print Center$(A$,80)
color %White,%Black
locate 22,1,1,6,7
goto About
'
'
' Should an unrecoverable error occure, we exit program here
'
NoJob:
'
print
color %Yellow,%Red
A$ = " * " + SLText$(15%) + " - "
print Center$(A$ + Fatal$,80);
color %LWhite,%Red
print Center$(SLText$(31%) + VN$ + SLText$(32%),80);
color %LCyan,%Blue
print Center$(SLText$(33%),80);
color %White,%Black
locate 22,1,1,6,7
A$ = KeyIn$
goto About
'
'
About:
'
print "Info = F1 (2 sec)"
print
delay 2
if inkey$ = chr$(0,59) then
  color %LWhite,%Black
  print "CCDFITS .BAS Version";CCDFitsV%
  color %White,%Black
  print "CCDFHEAD.INC Version";CCDFitsH%
  print "CCDFFUNC.INC Version";CCDFFuncV%
  print "CCDFPROC.INC Version";CCDFProcV%
  print "CCDFSUB1.INC Version";CCDFSub1V%
  print "CCDFSUB2.INC Version";CCDFSub2V%
  print "CCDFDAT .INC Version";CCDFDatV%
  print "CCDFF   .OBJ Version";CCDFFV%
  print "ERROR3  .INC Version";Error3V%
  print "IOTEXT  .INC Version";IOTextV%
  print "PB3GPUSE.PBU Version";PB3GPUseV%
  print "CLRSCRN. PBL Version";ClrScrnV%
  print "ARRAYEXT.PBL Version";ArrayExtV%
  print "RLCEGEN .OBJ Version";RlceGenV%
  print "PBSTACK .OBJ Version 100"
  print "PowerBASIC  Revision";PBVer%(SV$);SV$
  print
  color %LWhite,%Blue
  call HoroSoftLogo
  color %White,%Black
  while not instat : wend
  Q$ = inkey$
  call ClsEC
end if
cls
system
end
'
'
'===================================-<960318>-=========-<HoroSoft>-==========
'
