'970413 1832
'****************************************************************************
'* PROGRAM NAME: CCDFPROC.INC   DATE: 10. 2.1996   INCLUDE IN: CCDFITS .BAS *
'* LANGUAGE: Power BASIC 3.20   Robert S.Zale                               *
'* PURPOSE: Collection of procedures used in CCDFITS.BAS                    *
'* INCLUDE FILES: none - though they must be included/linked by main        *
'* INPUT FILES: none - files accessed must be opened by main                *
'* OUTPUT FILES: none - files written to must be opened by main             *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.15: 13-APR-97 Added Starlight Xpress and ST-7 formats         *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.14:  6-APR-96 Major improvements                              *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.13:  1-APR-96 Added gray scale at end of image                *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.12: 30-MAR-96 First release                                   *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.02: 22-MAR-96 Added TIFF to FITS                              *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.01: 10-FEB-96 Copied and extracted from CCDCPROC.INC          *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.00: 13-SEP-92 Subset of CCDPROC.INC                           *
'* ------------------------------------------------------------------------ *
'* COMMENTS:                                                                *
'* ======================================================================== *
'* AUTHOR: Hans-Ruedi H. Wernli, Pletschgasse, CH-3952 Susten, Switzerland. *
'****************************************************************************
'
'
CCDFProcV% = 115
'
'
' PROCEDURE  prints "" in green if ok, else text
'----------------------------------------------------------------------------
'
sub IssueOK(F%)
  if istrue F% then
    color %LGreen,%Black
    print ""
  else
    color %LRed,%Black
    print SLText$(18%)
  end if
end sub
'
'
' PROCEDURE creates FITS header. N2% is unchanged (saved in NS%)
'   Except for 3 planes images, a grayscale with 2+10 lines is added
'----------------------------------------------------------------------------
'
sub FitsHead(Card$(),V$,BP%,N0%,N1%,N2%,Hi??,Lo??,FH$,F$,E%)
  local C%,N%,X$,Y$
  C% = 1
  X$ = "SIMPLE  =" + space$(20) + "T / "
  Y$ = "Converted with " + V$
  Card$(C%) = left$(X$+Y$+space$(80),80)
  incr C%
  X$ = "BITPIX  =" + space$(16) + using$("#####",BP%) + " / "
  Card$(C%) = left$(X$+space$(80),80)
  incr C%
  X$ = "NAXIS   =" + space$(16) + using$("#####",N0%) + " / "
  if N0% = 3 then X$ = X$ + "Separate images for Red, Green and Blue"
  Card$(C%) = left$(X$+space$(80),80)
  incr C%
  X$ = "NAXIS1  =" + space$(16) + using$("#####",N1%) + " / "
  Card$(C%) = left$(X$+space$(80),80)
  incr C%
  X$ = "NAXIS2  =" + space$(16) + using$("#####",N2%) + " / "
  Card$( C%) = left$(X$+space$(80),80)
  incr C%
  if N0% = 3 then
    X$ = "NAXIS3  =" + space$(16) + using$("#####",N2%) + " / "
    Card$( C%) = left$(X$+space$(80),80)
    incr C%
  end if
  X$ = "DATAMIN =" + space$(13) + using$("#####.##",Lo??) + " / "
  Card$(C%) = left$(X$+space$(80),80)
  incr C%
  X$ = "DATAMAX =" + space$(13) + using$("#####.##",Hi??) + " / "
  Card$(C%) = left$(X$+space$(80),80)
  incr C%
  Card$(C%) = left$("OBJECT  = '"+F$+"'"+space$(70),80)
  incr C%
  Card$(C%) = left$("DATE    = '"+DateNorm$(E%)+"'"+space$(70),80)
  incr C%
  Card$(C%) = "DATE-OBS= " + space$(70)
  incr C%
  Card$(C%) = "ORIGIN  = " + space$(70)
  incr C%
  Card$(C%) = "INSTRUME= " + space$(70)
  incr C%
  Card$(C%) = "TELESCOP= " + space$(70)
  incr C%
  Card$(C%) = "OBSERVER= " + space$(70)
  incr C%
  Card$(C%) = left$("HISTORY = '"+FH$+"'"+space$(70),80)
  incr C%
  Card$(C%) = "COMMENT " + space$(72)
  incr C%
  Card$(C%) = "COMMENT " + space$(72)
  incr C%
  Card$(C%) = "COMMENT " + space$(72)
  incr C%
  Card$(C%) = "COMMENT " + space$(72)
  incr C%
  if N0% = 3 then
    X$ = "COMMENT  File offset Red   "
    Card$(C%) = X$ + using$("#########",2880)
    incr C%
    X$ = "COMMENT  File offset Green "
    Card$(C%) = X$ + using$("#########",N1%*N2%+2880)
    incr C%
    X$ = "COMMENT  File offset Blue  "
    Card$(C%) = X$ + using$("#########",N1%*N2%*2+2880)
    incr C%
  else
    Card$(C%) = "COMMENT " + space$(72)
    incr C%
    Card$(C%) = "COMMENT " + space$(72)
    incr C%
    Card$(C%) = "COMMENT " + space$(72)
    incr C%
  end if
  Card$(C%) = "END" + space$(77)
  incr C%
  for N% = C% to 36
    Card$(N%) = space$(80)
  next N%
end sub
'
'
' PROCEDURE updates Card with NAXIS2 and adds comment
'----------------------------------------------------------------------------
'
sub InsNaxCmt(Card$(),N2%,T%)
  local C%
  C% = 1
  do
    if istrue left$(Card$(C%),6) = "NAXIS2" then exit loop
    incr C%
  loop
  if C% < 36 then
    X$ = "NAXIS2  =" + space$(16) + using$("#####",N2%) + " / " + SLText$(T%)
    Card$(C%) = left$(X$+space$(80),80)
  end if
end sub
'
'
' PROCEDURE updates FITS-Head with ST4 image data - C% is NAXIS
'----------------------------------------------------------------------------
'
sub ST4Fits(Card$(),R$,C%)
  local A$,B$
  A$ = ST4Date$(R$) + " at " + ST4Time$(R$)
  Card$(8+C%) = left$("DATE-OBS= "+A$+space$(70),80)
  A$ = "CCD ST4, Integration time " + ST4Integ$(R$)
  B$ = ", Calibration factor " + ST4Calib$(R$)
  Card$(10+C%) = left$("INSTRUME= "+A$+B$+space$(30),80)
  A$ = "Focal length " + ST4FocLen$(R$)
  B$ = ", Aperture diameter " + ST4Apert$(R$)
  Card$(11+C%) = left$("TELESCOP= "+A$+B$+space$(40),80)
end sub
'
'
' PROCEDURE updates FITS-Head with ST6 or ST7 image data. C is NAXIS
'----------------------------------------------------------------------------
'
sub ST6Fits(Card$(),D$,C%,S%)
  local A$,B$,Z$
  Z$ = chr$(52 + S%)	'4+2=6, 4+3=7
  A$ = ST6Date$(D$) + " at " + ST6Time$(D$)
  Card$(8+C%) = left$("DATE-OBS= "+A$+space$(70),80)
  A$ = "CCD ST" + Z$ + ", Integration time " + FindVal$(D$,"Exposure =")
  B$ = ", Calibration factor " + FindVal$(D$,"Response_factor =")
  Card$(10+C%) = left$("INSTRUME= "+A$+B$+space$(30),80)
  A$ = "Focal length " + ST6FocLen$(D$)
  B$ = ", Aperture diameter " + ST6Apert$(D$)
  Card$(11+C%) = left$("TELESCOP= "+A$+B$+space$(40),80)
  A$ = FindVal$(D$,"Observer =")
  Card$(12+C%) = left$("OBSERVER= "+A$+space$(70),80)
end sub
'
'
' PROCEDURE gets NAXISis for PIC. S% is pointer in D$ and updated
'----------------------------------------------------------------------------
'
sub PicNax(S%,SD%,D$,N1%,N2%,BP%,Z&)
  S% = 69
  seek # %UnitAUX, SD%
  N1% = StrInt%(S%,D$)
  N2% = StrInt%(S%,D$)
  BP% = StrInt%(S%,D$)
  Z& = N1% * N2% * (BP% / 8) + SD%
end sub
'
'
' PROCEDURE displays FITS-Head
'-----------------------------------------------------------------------------
'
sub FHeadD(V$,O$,B$,Naxis%(2),I%)
  local N%,C$,D$
  print LogOn$(V$,%White,%Brown)
  print B$
  if Naxis%(I%,0) = 3 then print SLText$(66%);Naxis%(I%,3);SLText$(67%);"."
  if Confirm%(SLText$(59%)) then
    open O$ for input as # %UnitFIT
    while not eof(%UnitFIT)
      line input# %UnitFIT, D$
      color %Green,%Black
      print left$(D$,8);
      color %LGreen,%Black
      C$ = mid$(D$,9)
      if len(D$) > 79 then print C$; else print C$
      incr N%
      if N% = 17 then
        if isfalse Confirm%(SLText$(62%)) then exit loop
        N% = 1
      end if
    wend
    close %UnitFIT
  end if
  color %Cyan,%Black
  print SLText$(60%)
  if isfalse Confirm%(SLText$(61%)) and istrue FileHere%(O$) then kill O$
  print LogOn$(V$,%White,%Blue)
end sub
'
'
' PROCEDURE finds Hi and Lo values for PIC file
'----------------------------------------------------------------------------
'
sub FindHiLo(D$,BL%,BP%,Hi??,Lo??,CCA%())
  local H??,L??,H%
  call StrVar(CCA%(1),BL%,0%,D$)
  if BP% = 16 then
    H%  = BL% \ 2
    call AndMWA(CCA%(1),32767??,H%)
    L?? = MinWrd??(CCA%(1),H%)
    H?? = MaxWrd??(CCA%(1),H%)
    call MotInt(CCA%(1),H%)
  else
    H%  = BL%
    L?? = MinByt??(CCA%(1),H%)
    H?? = MaxByt??(CCA%(1),H%)
  end if
  if L?? < Lo?? then Lo?? = L??
  if H?? > Hi?? then Hi?? = H??
  call VarStr(CCA%(1),BL%,0%,D$)
end sub
'
'
' PROCEDURE  fills FITS file to full records
'----------------------------------------------------------------------------
'
sub FillFits(N1%,N2%,BP%)
  local S&,R%,S%
  S& = N1% * N2% * (BP% \ 8) + 2880
  R% = S& mod 2880
  if R% > 0 then
    S% = 2880 - R%
    R$ = string$(S%,chr$(0))
    put$ # %UnitFIT, R$
  end if
  close %UnitFIT
end sub
'
'
' PROCEDURE shows Move commands in line 23 & 24
'----------------------------------------------------------------------------
'
sub MovCmd(P%,E%)
  local A$,B$,Y%
  Y% = csrlin
  locate 22,1
  if istrue E% then
    A$ = "[TAB]["+chr$(31)+"]=down - ["+chr$(30)+"]=up"
  else
    A$ = "[TAB]["+chr$(31)+"]=nach unten - ["+chr$(30)+"]=hinauf"
  end if
  color %LWhite,%Blue
  select case P%
    case 1
      print Center$(A$+" - "+SLText$(46%),80);
    case 2
      print Center$(A$+" - "+SLText$(46%)+" - "+SLText$(47%),80);
    case 3
      print Center$(A$+" - "+SLText$(47%),80);
  end select
  locate 23,1
  color %LWhite,%Cyan
  print Center$(SLText$(48%),80);
  locate 24,1
  color %White,%Black
  call ClrLine
  locate Y%,1
end sub
'
'
' PROCEDURE  shows Edit commands in line 23
'----------------------------------------------------------------------------
'
sub EdtCmd(E%)
  local Y%,A$
  Y% = csrlin
  color %LWhite,%Blue
  if istrue E% then
    A$ = "["+chr$(17)+"]=left - ["+chr$(16)+"]=right  OVERWRITE"
  else
    A$ = "["+chr$(17)+"]=links - ["+chr$(16)+"]=rechts  BERSCHREIBEN"
  end if
  locate 22,1
  color %LWhite,%Blue
  print Center$(A$,80);
  color %Yellow,%Cyan
  print Center$(SLText$(50%),80);
  color %LWhite,%Red
  print Center$(SLText$(49%),80);
  color %White,%Black
  locate Y%,1
end sub
'
'
' PROCEDURE  distributes FITS-Header into 36 cards
'----------------------------------------------------------------------------
'
sub FillCards(Card$(),ACrd$(),R$)
  local F%,I%
  for F% = 1 to 36
    I% = (F%-1) * 80 + 1
    Card$(F%) = mid$(R$,I%,80)
    ACrd$(F%) = Card$(F%)
  next F%
end sub
'
'
' PROCEDURE  displays one page = 12 Cards
'----------------------------------------------------------------------------
'
sub DisPage(P%,ACrd$())
  local F%,C%
  locate 5,1
  C% = (P%-1) * 12
  for F% = 1 to 12
    print ACrd$(F%+C%);
  next F%
  locate 5,1
end sub
'
'
' PROCEDURE  removes old and inserts new cursorline
'----------------------------------------------------------------------------
'
sub InsCurs(L%,O%,ACrd$())
  local A%,B%
  A% = L%
  B% = O%
  while A% > 12
    A% = A% - 12
  wend
  while B% > 12
    B% = B% - 12
  wend
  A% = A% + 4
  B% = B% + 4
  locate B%,1
  color %White,%Black
  print ACrd$(O%);
  locate A%,1
  color %Black,%White
  print ACrd$(L%);
  color %White,%Black
  locate A%,1
end sub
'
'
' PROCEDURE  prints edit cursor
'----------------------------------------------------------------------------
'
sub EdtCurs(C%,O%,A$)
  local N$,O$
  N$ = mid$(A$,C%,1)
  O$ = mid$(A$,O%,1)
  color %LWhite,%Black
  locate 18,O%
  print mid$(A$,O%,1);
  color %LWhite,%White
  locate 18,C%
  print mid$(A$,C%,1);
  locate 18,C%
  color %LWhite,%Black
end sub
'
'
' PROCEDURE copies editted cards into actual cards and inserts END if
'   necessary
'----------------------------------------------------------------------------
'
sub FindEnd(Card$())
  local N%,F%
  for N% = 1 to 36
    if left$(Card$(N%),3) = "END" then
      Card$(N%) = space$(80)
      exit for
    end if
  next N%
  for F% = 36 to 1 step -1
    if isfalse remove$(Card$(F%)," ") = "" then exit for
  next F%
  if F% < 1 then F% = 1 else incr F%
  Card$(F%) = "END" + space$(77)
end sub
'
'
' PROCEDURES FOR TIFF HEAD ANALYSIS
'============================================================================
'
' PROCEDURE  returns one tag. Tagses(1,n)=Typ, Tagses(2,n)=Length
'   Tagses(3,n)=Pointer where n=Tag number
'----------------------------------------------------------------------------
'
sub GetTag(R$,M%,Tagses???(2),F%)
  local P%,T%,D$,Z$
  P%  = (F%-1) * 12 + 1
  D$  = mid$(R$,P%,12)
  Z$  = left$(D$,2)
  T%  = ShortInt??(Z$,M%)
  Z$  = mid$(D$,3,2)
  Tagses???(1,T%) = ShortInt??(Z$,M%)
  Z$  = mid$(D$,5,4)
  Tagses???(2,T%) = LongInt???(Z$,M%)
  Z$ = right$(D$,4)
  Tagses???(3,T%) = LongInt???(Z$,M%)
end sub
'
'
' PROCEDURE returns T%urns, BytesPerB%lock and R%estBytes from BitPix,
'   Naxis1 and S???tripByteCount
'----------------------------------------------------------------------------
'
sub BlockLen(B%,R%,T%,BP%,N1%,S???)
  local A%,L%
  A% = N1% * (BP% \ 8)
  L% = B% \ A%
  B% = L% * A%
  T% = (S??? \ A%) \ L%
  R% = S??? - (L% * T% * A%)
end sub
'
'
'===============================-<960321>-============-<HoroSoft>-===========
