'960412 1430
'****************************************************************************
'* PROGRAM NAME: CCDLSCVT.BAS   DATE:  5.12.1993   INCLUDE IN: is main      *
'* LANGUAGE: Power BASIC 3.20  (Robert S.Zale)                              *
'* PURPOSE: Converts between Lynxx and ST4 picture formats.                 *
'* INCLUDE FILES: ERROR3  .INC  Fatal error handler                         *
'*                IOTEXT  .INC  Set of Text and File functions              *
'*                CCDLPROC.INC  Procedures and a function                   *
'*                CCDLDAT .INC  English and German Texts                    *
'*                CCDLSUB .INC  Input/Output subroutines                    *
'* OBJECT FILES:  PBSTACK. OBJ  Power BASIC 2 & 3 Stack handler for objects *
'*                ARRAYEXT.PBL  Advanced array manipulation functions)      *
'*                CLRSCRN .PBL  Collection of cool screen sweeper           *
'*                CCDLL   .OBJ  procedures for CCDUTIL and CCDCONVT         *
'* UNIT FILES:    PB3GPUSE.PBU  General Purpose Functions and Procedures    *
'* INPUT FILES:   filename.LNX  Lynxx image                                 *
'*                filename.ALI  Artificial Lynxx Image                      *
'*                filename.ST4  ST-4 image                                  *
'*                filename.ASI  Artificial ST-4 Image                       *
'* OUTPUT FILES:  filename.ALI  Artificial Lynxx Image                      *
'*                filename.ASI  Artificial ST-4 Image                       *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.44  8-APR-96: Added cool screen clearer                       *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.43 10-FEB-96: Made it FreeWare                                *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.42  2-SEP-95: Recompiled from PB3.10 to PB3.20                *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.41 17-FEB-95: Must now be licenced to run always.             *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.33  1-JUN-94: Introduced Version CCDLSCvtV% = 133             *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.23 31-DEC-93: Removed BYT-File option                         *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.13 27-DEC-93: Included Run Length Count Encoding              *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.03  5-DEC-93: Derived from CCDCONVT                           *
'* ------------------------------------------------------------------------ *
'* COMMENTS: Program on Robert Glaisen's suggestion. ST4 images are         *
'*           flipped, i.e. mirrored and put upside down and stretched when  *
'*           converted to Lynxx format for best appearance. Lynxx images    *
'*           are also flipped, then compressed for ST4.                     *
'* ------------------------------------------------------------------------ *
'* NOTE: For file formats, see listing of CCDUTIL.                          *
'* ======================================================================== *
'* AUTHOR: Hans-Ruedi H. Wernli, Pletschgasse, CH-3952 Susten, Switzerland. *
'****************************************************************************
'
'
' COMPILER DIRECTIVES
'----------------------------------------------------------------------------
'
'$alias data as "DSEG"	'or data segment name as used in .OBJ
'$code seg "CODE"	'or code segment name as used in .OBJ
'$huge			'for arrays > 64K
$cpu 8086		'8086/80286/80386
$debug map off		'on/off
$debug path off		'on/off
$debug unit off		'on/off
$debug pbdebug off	'on/off
$dim array		'all/array/none
$dynamic		'$static
$error all off		'bounds/numeric/overflow/stack/all on/off
$event off		'on/off
$float procedure	'emulate/npx/procedure
$lib com-,lpt-,graph-,cga-,ega-,vga-,herc-,fullfloat-,iprint-
$optimize speed		'size/speed
$option cntlbreak off	'cntlbreak/gosub on/off
$stack 2048		'1536...32767
$com 0			'0...32767
$sound 1		'1...4096
$string 4		'1/2/4/8/16/32
$compile exe		'memory/exe/chain/unit
'
'
' LANGUAGE SELECTION AND VERSION NUMBER
'----------------------------------------------------------------------------
'
VN$      = "CCDLSCVT V.1.44"
CCDLSCvtV% = 144
on error goto ErrorHandling
%True   = -1
%False  =  0
Chroma% = %True
shared Idiom%()
dim Idiom%(0:2)			'0:E=true/D=false 1:SL-text-, 2:ML text-offs
English% = ComLinLan%(command$,"CCDTOOLS: "+VN$)
if English% then
  Idiom%(0) = %True		'English
  Idiom%(1) = 0			'SL offset
  Idiom%(2) = 0			'ML offset
  Version$ = "Lynxx <--> ST4 Converter & Encoder" + VN$
else
  Idiom%(0) = %False		'Deutsch
  Idiom%(1) = 50		'SL offset
  Idiom%(2) = 10		'ML offset
  Version$ = "Lynxx <--> ST4 Konverter & Kodierer" + VN$
end if
'
'
' CONSTANTS COLORS
'-----------------------------------------------------------------------------
'
%Black     =  0        'COLORS, dark
%Blue      =  1
%Green     =  2
%Cyan      =  3
%Red       =  4
%Pink      =  5
%Brown     =  6
%White     =  7
%Gray      =  8        'COLORS, light
%LBlue     =  9
%LGreen    = 10
%LCyan     = 11
%LRed      = 12
%LPink     = 13
%Yellow    = 14
%LWhite    = 15
'
%DOSRead  = 63		'DOS function used at Encode/Decode for ST4 and ASI
%DOSWrite = 64		'  EDCST4
'
'
' OTHER CONSTANTS AND VARIABLES DECLARATIONS
'----------------------------------------------------------------------------
'
%LMN       = 31680     'eLeMeNts in arrays
%UnitAUX   = 1         'Only one file is open at a time
'
Chroma%  = %True	'TRUE if colors available (for ERROR.INC)
on error goto ErrorHandling
'
'
' Function and Procedure Declaration in external object: CLRSCRN.PBL
'----------------------------------------------------------------------------
'
declare function ClrScrnV%()
declare sub ClsLC()
declare sub ClsCL()
declare sub HoroSoftLogo()
'
'
' Function and Procedure Declaration in external object: ARRAYEXT.ASM/OBJ
'----------------------------------------------------------------------------
'
declare function ArrayExtV%()
declare sub SubCWL(integer,integer,integer,integer)
declare sub ShrWrd(integer,integer)
declare sub XDimWA(integer,integer,integer,integer)
declare sub CPYW2B(integer,integer,integer)
declare sub CPYB2W(integer,integer,integer)
declare sub CPYW2W(integer,integer,integer)
declare function MINWRD%(integer,integer)
declare function MAXWRD%(integer,integer)
'
'
' Function and Procedure Declaration in external units: PB3GPUSE.INC/PBU
'----------------------------------------------------------------------------
'
declare function PB3GPUseV%()
declare function PBVer%(string)
declare function EscRet%()
declare function AddSlash$(string)
declare function Center$(string,integer)
declare function LogOn$(string,integer,integer)
declare function DateNorm$(integer)
declare function KeyIn$()
declare function ComLinLan%(string,string)
declare function IOPath$(string,string,string)
declare function IOFile$(integer,integer,string)
declare function Confirm%(string)
declare function Redo%(string)
declare function FileHere%(string)
declare function SizeOk%(string,long)
declare function ByteBin$(integer)
declare function Outch$(string)
declare sub ClrLine()
declare sub Illegal(string,string)
'
'
' Function and Procedure Declaration in external units: CCDLL.ASM/OBJ
'----------------------------------------------------------------------------
'
declare function CCDLLV%()
declare sub DecLnx(integer,integer)
declare sub EncLnx(integer,integer,integer)
declare sub EDCST4(integer,string,integer,integer)
declare sub FLIPXY(integer,integer)
declare sub MltPly(integer,integer)
'
'
' FILES TO BE LINKED IN
'-----------------------------------------------------------------------------
'
$link "PBSTACK.OBJ"
$link "ARRAYEXT.PBL"
$link "CLRSCRN.PBL"
$link "CCDLL.OBJ"
$link "PB3GPUSE.PBU"
'
'
' FILES TO BE INCLUDED
'-----------------------------------------------------------------------------
'
$include "CCDLPROC.INC"
$include "IOTEXT.INC"
$include "CCDLSUB.INC"
$include "CCDLDAT.INC"
$include "ERROR3.INC"
'
'
' VARIABLES SETUP
'-----------------------------------------------------------------------------
'
GPCR$ = "(c) 1993-96 by H.-R.H. Wernli, CH-3952 Susten, Switzerland"
STCR$ = "Processed with " + VN$ + "; " + "H.-R.H.Wernli, CH-3952 Susten."
Nill$ = ""
InpPath$ = curdir$
OutPath$ = curdir$
'
Z% = fix(fre(-1)/1024)
Fatal$ = SLText$(6%) + str$(219-Z%) + "Kb"
if Z% < 219 then goto NoJob	'Needs about 307Kb
'
dim IOExt$(1:4)
IOExt$(1) = "LNX"
IOExt$(2) = "ALI"
IOExt$(3) = "ST4"
IOExt$(4) = "ASI"
dim FSize&(1:4)
FSize&(1) = 47520
FSize&(2) = 47520
FSize&(3) = 31872
FSize&(4) = 31872
dim BatchI$(1:16)
dim BatchO$(1:16)
dim Batch%(1:16)
dim CCD%(1:165,1:192)
dim CCA%(1:31872)	'-- 192*166 to hold ST4 images
dim CCQ%(1:192,1:165)
'
'
' START OF PROGRAM
'============================================================================
'
' Setup of Filespec string-array, flags array and use sequence array
'
Virgin% = %True
'
'----------------------------------- Log on ---------------------------------
'
print LogOn$(Version$,%Cyan,%Black)
print Describe$
'
'
'=============================-<Start of other Conversion Loop>-=============
do
  for N% = 1 to 16
    BatchI$(N%) = Nill$
    BatchO$(N%) = Nill$
    Batch%(N%)  = %False
  next N%
'
' Log On and input option gathering: 1...5
'
  print LogOn$(Version$,%Cyan,%Black);
  print Describe$
  Y% = csrlin
  locate Y%,1
  color %LCyan,%Black
  print Center$(SLText$(5%),80)
  color %Cyan,%Black
  call MLText(2%,15%,"tf")
'
'-|-------------------------------- Start of Input Option Loop --------------
  do
    InpOpt% = Prompt%(20%,Q$)
    if InpOpt% > 0 and InpOpt% < 6 then exit loop
    print Outch$(Q$)
  loop
  select case InpOpt%
    case 1, 2
      OutOpt% = 4
    case 3, 4
      OutOpt% = 2
    case 5
      exit loop
  end select
'
'.|................................ End of Input Option Loop ................
'
'
'-- Prepare file extensions
'
  InpExt$ = IOExt$(InpOpt%)
  OutExt$ = IOExt$(OutOpt%)
'
'
'-- Get Output path
'
  print LogOn$(Version$,%Brown,%Black);
  print Describe$
  I% = %False
  gosub GetDirPath
  OutPath$ = UsePath$
  if Virgin% then InpPath$ = UsePath$	'-- only the first time
'
'-|------------------------- Start of Input Path Loop ----------------------
  do
    print LogOn$(Version$,%Green,%Black);
    I% = %True
    gosub GetDirPath			'-- Get input path
    InpPath$ = UsePath$
    A% = ListFiles%(InpPath$,InpExt$,English%)
    if A% then exit loop
    beep
    Fatal$ = SLText$(3%)
    if not Redo%(SLText$(39%)) then goto NoJob
  loop
'.|............................... End of Input Path Loop ...................
'
  Y? = csrlin
  if Y? > 21 then
    for N% = 1 to 7
      print
    next N%
  end if
  locate 21,1
'
'-- Input file gathering
'
  color %Black,%Cyan
  print Center$(SLText$(2%),80)
  color %Cyan,%Black
  Y% = csrlin
  for N% = 1 to 16
    locate Y%,1
    print space$(80);
    locate Y%,1
    A$ = IOFile$(English%,%False,SLText$(36%))
    if instr(A$,"END") then exit for
    BatchI$(N%) = InpPath$ + StripExt$(A$) + "." + InpExt$
    BatchO$(N%) = OutPath$ + StripExt$(A$) + "." + OutExt$
  next N%
  NoFiles% = N% - 1
'
'-- Filespec and filesize validification, construction of output filespecs
'
  print Logon$(Version$,%Cyan,%Blue);
  color %LCyan,%Black
  print Center$(SLText$(4%)+Space$(6)+"("+SLText$(16%)+")",80)
  for N% = 1 to NoFiles%
    color %White,%Black
    print "[";
    print using "##";N%;
    print "] ";
    color %Green,%Black
    print BatchI$(N%);" ";
    if not dir$(BatchI$(N%)) = "" then
      open BatchI$(N%) for input as # %UnitAUX
      if lof(%UnitAUX) = FSize&(InpOpt%) then
	color %LGreen,%Black
	print ""
	Batch%(N%) = %True
      else
	color %LRed,%Black
	print SLText$(18%);lof(1)
      end if
      close %UnitAUX
    else
      color %LRed,%Black
      print SLText$(19%)
      Batch%(N%) = %False
    end if
  next N%
'
'-- Moving filespecs such that invalid ones are skipped
'
  Cnt% = 0
  for N% = 1 to NoFiles%
    if Batch%(N%) then
      incr Cnt%
      BatchI$(Cnt%) = BatchI$(N%)
      BatchO$(Cnt%) = BatchO$(N%)
    end if
  next N%
  NoFiles% = Cnt%
  Q$ = KeyIn$
'
' Verify to overwrite or skip existing files, if any - but only, if not
' Lynxx - Lynxx conversion
'
  print LogOn$(Version$,%White,%Red)
  color %Cyan,%Black
  print Center$(SLText$(11%),80)
  for N% = 1 to NoFiles%
    if not dir$(BatchO$(N%)) = "" then
      color %Green,%Black
      print BatchO$(N%);" ";
      color %LRed,%Black
      print SLText$(8%)
      if not Confirm%(SLText$(9%)) then Batch%(N%) = %False
    end if
  next N%
  color %LWhite,%Black
'
' Now we can start doing the job. We select the input menu and for each one,
' do all output possibilities.
'
  call ClsLC
  print LogOn$(Version$,%White,%Blue)
  color %Cyan,%Black
  print SLText$(17%);
  Y? = csrlin
  X? = pos(Y?)
  for N% = 1 to NoFiles%
    locate Y?,X?
    color %LCyan,%Black
    print using "###";NoFiles%-N%+1
    color %Cyan,%Black
    print
    ST4Info$ = chr$(190) + space$(190) + chr$(0)
    if Batch%(N%) then on InpOpt% gosub ILNX, ILNX, IST4, IST4
    for F? = Y?+1 to 20
      locate F?,1
      call ClrLine
    next F?
  next N%
  locate Y?,X?
  color %LCyan,%Black
  print using "###";NoFiles%-N%+1
  color %Cyan,%Black
'
' Redo all with different input and output menus
'
  print
  Virgin% = %False
  if not Confirm%(SLText$(7%)) then exit loop
loop
'
'::::::::::::::::::::::::::::::: End of other Conversion Loop :::::::::::::::
'
' That's it - we're going to log off
'
LogOff:
'
print LogOn$(Version$,%White,%Blue);
print Describe$
color %LWhite,%Green
A$ = VN$ + SLText$(30%)
print Center$(A$,80)
color %LWhite,%Red
A$ = VN$ + "; " + GPCR$
print Center$(A$,80)
color %White,%Black
locate 22,1,1,6,7
goto About
'
'
' Should an unrecoverable error occur, we exit program here
'
NoJob:
'
print
color %Yellow,%Red
A$ = " * " + SLText$(15%) + " - "
print Center$(A$ + Fatal$,80);
color %LWhite,%Red
print Center$(SLText$(31%) + VN$ + SLText$(32%),80);
color %LCyan,%Blue
print Center$(SLText$(33%),80);
color %White,%Black
locate 22,1,1,6,7
A$ = KeyIn$
goto About
'
'
About:
'
print "Info = F1 (2 sec)"
delay 2
if inkey$ = chr$(0,59) then
  print
  color %LWhite,%Black
  print "CCDLSCVT.BAS Version";CCDLSCvtV%
  color %White,%Black
  print "CCDLPROC.INC Version";CCDLProcV%
  print "CCDLSUB .INC Version";CCDLSubV%
  print "CCDLDAT .INC Version";CCDLDatV%
  print "CCDLL   .OBJ Version";CCDLLV%
  print "IOTEXT  .INC Version";IOTextV%
  print "ERROR3  .INC Version";Error3V%
  print "PB3GPUSE.PBU Version";PB3GPUseV%
  print "CLRSCRN .PBL Version";ClrScrnV%
  print "ARRAYEXT.PBL Version";ArrayExtV%
  print "PBSTACK .OBJ Version 100"
  print "PowerBASIC  Revision";PBVer%(SV$);SV$
  print
  color %LWhite,%Blue
  call HoroSoftLogo
  color %White,%Black
  while not instat : wend
  Q$ = inkey$
  call ClsCL
end if
cls
system
end
'
'
'===================================-<931206>-=========-<HoroSoft>-==========
'
