'960412 1431
'****************************************************************************
'* PROGRAM NAME: CCDPRINT.BAS   DATE: 12. 2.1993   INCLUDE IN: is Main!     *
'* LANGUAGE: Power BASIC 3.20  (Robert S.Zale)                              *
'* PURPOSE: Additional Tools for CCD Astro Camera Purposes.                 *
'* INCLUDE FILES: ERROR3  .INC  Fatal error handler                         *
'*                IOTEXT  .INC  Set of Text and File functions              *
'*                CCDPFUNC.INC  Dedicated Functions.                        *
'*                CCDPPROC.INC  Dedicated Procedures.                       *
'*                CCDPSUB .INC  Dedicated Subroutines.                      *
'*                CCDPDAT .INC  Text data for CCDPRINT.                     *
'* OBJECT FILES:  PBSTACK .OBJ  Power BASIC 2 & 3 Stack handler for objects *
'*                ARRAYEXT.PBL  Byte, word and integer array functions.     *
'*                CLRSCRN .PBL  Hot screen clearer collection               *
'*                CCDPP   .OBJ  Assembly Routines with external procedures  *
'*                CCDUP   .OBJ  Assembly routines for large images          *
'*                RLCEGEN .OBJ  Run Length Count Encoding Generator         *
'* UNIT FILES:    PB3GPUSE.PBU  General Purpose Functions and Procedures    *
'* INPUT FILES:   filename.BYT  8-bit image for displaying.                 *
'*                filename.Bxx  where xx [00;99] BYT-Type files             *
'*                filename.BYR  BYG & BYB - Color files RGB                 *
'*                filename.PCX  PCX picture files                           *
'*                filename.DOT  Dotogramm-type images                       *
'*                filename.Dxx  where xx [00;99] Dot-Type files             *
'*                filename.CGE  Gallery file                                *
'* OUTPUT FILES:  filename.Ixx  where xx [00;99] PCX-Format Image file      *
'*                filename.Pxx  where xx [00;99] PCL-3 Printer file         *
'*                filename.BYC  RGB-Composite Byte-Type file                *
'*                filename.Txx  TIFF file in grayscale                      *
'*                filename.PCL  CGE-file for print                          *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.74   8-APR-96: Added supercool screen clear                   *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.73  10-FEB-96: Made it FreeWare                               *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.72   2-SEP-95: Recompiled from PB3.10 to PB3.20               *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.71  17-FEB-95: Must now be registered to run always           *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.63   7-JUL-94: Added FITS 6-Plane from BYR/BYG/BYB            *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.53   1-JUN-94: Overwholed RLCE-Encoder, is now a function.    *
'*                 Introduced Version CCDPrintV% = 253                      *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.43  10-FEB-94: Added TIFF write and CGE read.                 *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.33  31-DEC-93: BYT and DOT files are RLCE encoded             *
'*                11-JAN-94: Bug fix for PCX ---> RGB                       *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.23  24-DEC-93: Minimal revision to be able to use PB 3.00c    *
'*                compiler instead if PB 2.10e                              *
'*                Added also RLCE BIE and DOT read capability               *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.13  24-DEC-93: Minimal revision to be able to use PB 3.00c    *
'*                compiler instead if PB 2.10e                              *
'*                Added also RLCE BIE and DOT read capability               *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.02   1-JUN-93: Preparing for PB2, Series 2                    *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.00  12-FEB-93: Start                                          *
'* ------------------------------------------------------------------------ *
'* COMMENTS:                                                                *
'* ======================================================================== *
'* AUTHOR: Hans-Ruedi H. Wernli, Pletschgasse, CH-3952 Susten, Switzerland. *
'****************************************************************************
'
'
' COMPILER DIRECTIVES
'----------------------------------------------------------------------------
'
'$alias data as "DSEG"	'or data segment name as used in .OBJ
'$code seg "CODE"	'or code segment name as used in .OBJ
'$huge			'for arrays > 64K
$cpu 8086		'8086/80286/80386
$debug map off		'on/off
$debug path off		'on/off
$debug unit off		'on/off
$debug pbdebug off	'on/off
$dim array		'all/array/none
$dynamic		'$static
$error all off		'bounds/numeric/overflow/stack/all on/off
$event off		'on/off
$float procedure	'emulate/npx/procedure
$lib com-,lpt-,graph+,cga+,ega+,vga+,herc-,fullfloat-,iprint-
$optimize speed		'size/speed
$option cntlbreak off	'cntlbreak/gosub on/off
$stack 4096		'1536...32767
$com 0			'0...32767
$sound 1		'1...4096
$string 32		'1/2/4/8/16/32
$compile exe		'memory/exe/chain/unit
'
'
' LANGUAGE SELECTION AND VERSION NUMBER
'----------------------------------------------------------------------------
'
VN$      = "CCDPRINT V.2.74"
CCDPrintV% = 274
on error goto ErrorHandling
%True   = -1
%False  =  0
Chroma% = %True
shared Idiom%()
dim Idiom%(0:2)			'0:E=true/D=false 1:SL-text-, 2:ML text-offs
English% = ComLinLan%(command$,"CCDTOOLS: "+VN$)
if English% then
  Idiom%(0) = %True		'English
  Idiom%(1) = 0			'SL offset
  Idiom%(2) = 0			'ML offset
  Version$ = "CCD Image File Print-Tool " + VN$
else
  Idiom%(0) = %False		'Deutsch
  Idiom%(1) = 200		'SL offset
  Idiom%(2) =  50		'ML offset
  Version$ = "CCD Bild Datei Druck-Werkzeug " + VN$
end if
'
'
' VARIABLES SETUP and VIDEO TEST - SHARED VARIABLES AS WELL
' -----------------------------------------------------------------------------
'
%VBIOS = 16
%EGA   =  9
%VGA   = 12
%AX    =  1
%BX    =  2
screen 0
A% = BestVideo%
select case A%
  case 9
    EGAHere% = %True
    ScrMode% = 9
  case 12
    EGAHere% = %True
    VGAHere% = %True
    ScrMode% = 9
  case else
    EGAHere% = %False
    VGAHere% = %False
    ScrMode% = 0
end select
Chroma%  = %True
on error goto ErrorHandling
Nill$ = ""
OldPath$ = curdir$
'
'
' CONSTANTS COLORS
'-----------------------------------------------------------------------------
'
%Black     =  0        'COLORS, dark
%Blue      =  1
%Green     =  2
%Cyan      =  3
%Red       =  4
%Pink      =  5
%Brown     =  6
%White     =  7
%Gray      =  8        'COLORS, light
%LBlue     =  9
%LGreen    = 10
%LCyan     = 11
%LRed      = 12
%LPink     = 13
%Yellow    = 14
%LWhite    = 15
'
%LMN       = 31680      'eLeMeNts in arrays
%UnitAUX   =  1		'Input file
%UnitY00   =  2         'Byte file
%UnitP00   =  3         'PCL file
%UnitI00   =  4         'Image file (PCX)
%UnitT00   =  5		'Image file (TIFF)
%UnitPCL   =  3		'PCL File
%UnitRGB   =  4		'Create RGB from PCX
'
'
' Function and Procedure Declaration in external object: CLRSCRN.PBL
'----------------------------------------------------------------------------
'
declare function ClrScrnV%()
declare sub ClsVB()
declare sub HoroSoftLogo()
'
'
' Function and Procedure Declaration in external object: ARRAYEXT.ASM/OBJ
'----------------------------------------------------------------------------
'
declare function ArrayExtV%()
declare sub XDimBA(integer,integer,integer,integer)
declare sub FILLWC(integer,integer,integer)
declare sub ADDBAS(integer,integer,integer,integer)
declare sub ORWAS (integer,integer,integer)
declare sub CPYW2W(integer,integer,integer)
declare function MAXBYT%(integer,integer)
'
'
' Function and Procedure Declaration in external units: PB3GPUSE.INC/PBU
'----------------------------------------------------------------------------
'
declare function PB3GPUseV%()
declare function PBVer%(string)
declare function EscRet%()
declare function AddSlash$(string)
declare function Center$(string,integer)
declare function LogOn$(string,integer,integer)
declare function DateNorm$(integer)
declare function KeyIn$()
declare function ComLinLan%(string,string)
declare function IOPath$(string,string,string)
declare function IOFile$(integer,integer,string)
declare function Confirm%(string)
declare function Redo%(string)
declare function FileHere%(string)
declare function SizeOk%(string,long)
declare function ByteBin$(integer)
declare function Outch$(string)
declare sub ClrLine()
declare sub Illegal(string,string)
'
'
' Function and Procedure Declaration in external units: CCDUP.ASM/OBJ
'----------------------------------------------------------------------------
'
declare function CCDUPV%()
declare sub VarUP(integer)
declare sub OnePCL(integer,integer,string,integer)
declare sub RLEscn(string,integer,integer)
declare sub RLElin(string,integer,integer)
declare function BytWrd%(integer,integer)
declare function NibWrd%(integer,integer,integer,integer)
'
'
' Function and Procedure Declaration in external units: CCDPP.ASM/OBJ
'----------------------------------------------------------------------------
'
declare function CCDPPV%()
declare sub VarPP(integer)
declare sub LitPCL(integer,integer,string,integer)
declare sub LitScn(string,integer,integer)
declare sub LitLin(string,integer,integer)
declare sub DotPCL(string,integer)
declare sub DotScn(string,integer)
declare sub DotRLE(string,integer)
declare sub ProPCL(string,integer,integer)
declare sub IsoPCL(string,integer,integer)
declare sub IsoIso(string,integer)
declare sub ToRLLE(string,integer,integer)
declare sub PcxAdj(integer,integer)
declare sub PcxPal(integer,integer)
declare sub InsByt(integer,integer,integer,string)
declare sub FreAry(integer,integer)
declare sub BitByt(integer,integer,integer)
declare sub E1Gray(integer,string)
declare sub S1Gray(integer,string)
declare sub Raster(integer,integer)
declare sub VGACol(integer,integer)
declare sub StuffV(integer)
declare sub ColAss(integer,integer,integer,integer)
declare sub BytStr(string,integer,integer)
declare sub Third(integer,integer)
declare sub Sixth(integer,integer,integer)
declare sub Promot(integer,integer,string)
declare sub Make4B(integer,integer,integer)
declare sub NiByte(integer,integer)
declare sub ByteNi(integer,integer)
declare sub CgeReo(integer,integer)
declare function BitAnd%(integer,integer)
declare function RdFits%(integer,integer)
declare function WrTiff%(integer,integer,integer)
declare function TiffWr%(string,integer,integer,integer,integer,integer)
'
'
' Function and Procedure Declaration in external units: RLCEGEN.ASM/OBJ
'----------------------------------------------------------------------------
'
declare function RlceGenV%()
declare sub AryVid(integer)
declare function EncBie%(integer,integer,word)
declare function DecBie%(integer,integer)
declare function RdBie%(integer,integer,word)
declare function WrBie%(integer,integer)
'
'
' FILES TO BE LINKED IN
'-----------------------------------------------------------------------------
'
$link "PBSTACK.OBJ"
$link "ARRAYEXT.PBL"
$link "CLRSCRN.PBL"
$link "CCDUP.OBJ"
$link "CCDPP.OBJ"
$link "RLCEGEN.OBJ"
$link "PB3GPUSE.PBU"
'
'
' FILES TO BE INCLUDED
'-----------------------------------------------------------------------------
'
$include "IOTEXT.INC"
$include "CCDPFUNC.INC"
$include "CCDPPROC.INC"
$include "CCDPSUB.INC"
$include "CCDPDAT.INC"
$include "ERROR3.INC"
$segment
'
'
' VARIABLES SETUP
'-----------------------------------------------------------------------------
'
GPCR$ = "(c) 1993-96 by H.-R.H. Wernli, CH-3952 Susten, Switzerland"
STCR$ = "Processed with " + VN$ + "; (c) H.-R.H.Wernli, CH-3952 Susten."
InpPath$ = curdir$
OutPath$ = curdir$
'
dim FilExt$(1:5)
FilExt$(1) = "BYT"	'FilExt$(2/3/4/5): Y00/P00/I00/T00 set in Menu loop
'
Z% = fix(fre(-1)/1024)
Fatal$ = SLText$(81%) + str$(191-Z%) + "Kb"
if Z% < 191 then goto NoJob	'Needs about 409Kb
'
dim CCA%(1:%LMN/2)		'original data frame - PCX-Red - CGE>TIFF
dim CCP%(1:%LMN/2)		'active, printer frame - PCX-Green
dim CCD%(1:%LMN/2)              'Doto-type frame - PCX-Blue
dim CCI%(1:%LMN/2)		'image file - PCX-Intensity
dim CCB%(1:%LMN/2+5)		'Auxilliary
dim CCG%(1:1760)		'Gallery file
dim PXH%(0:12)			'PCX Head
dim FH$(1:16)			'Part of FITS-Head
FH$( 1) = "SIMPLE  =" + space$(20) + "T"
FH$( 2) = "BITPIX  =" + space$(20) + "8"
FH$( 3) = "NAXIS   =" + space$(20) + "3"
FH$( 4) = "NAXIS1  =" + space$(18) + "192"
FH$( 5) = "NAXIS2  =" + space$(18) + "165"
FH$( 6) = "NAXIS3  =" + space$(20) + "6"
FH$( 7) = "ORIGIN    '" + STCR$ + "'"
FH$( 8) = "DATE      '(mm-dd-yy) " + date$ + "'"
FH$( 9) = "HISTORY   '"
FH$(10) = "COMMENT   'NAXIS3: "
FH$(11) = "1 = Red_30% + Green_59% + Blue_11%'"
FH$(12) = "2 = Red_100%'"
FH$(13) = "3 = Green_100%'"
FH$(14) = "4 = Blue_100%'"
FH$(15) = "5 = Red_33.3% + Green_33.3% + Blue_33.3%'"
FH$(16) = "6 = (Red*36/51) + (Green*6/51) + (Blue*1/51)'"
FulPal$ = string$(768,chr$(0))	'Extended Palette data for PCX 3
StdPal$ = string$(48,chr$(0))	'Standard Palette Data for PCX 2
TiffQ$ = string$(23760,chr$(0))	'can hold 1/4 of 24-bit image
dim ColScale%(0:7)		'color raster scale
dim GrayScale%(0:15)		'grayscale
call ArraySetUp(ColScale%(),GrayScale%())
'
'
' START
'============================================================================
'
Virgin% = %True
'
'----------------------------- Log on ---------------------------------------
'
print LogOn$(Version$,%Cyan,%Black)
print Describe$
'
'
'===================-<Start of Main Loop>-===================================
'
do
  FilExt$(2) = ".Y00"
  FilExt$(3) = ".P00"
  FilExt$(4) = ".I00"
  FilExt$(5) = ".T00"
  PCXState%  = 0		'0: closed, 1: open, 2: intus
  call FillWC(CCA%(1),0%,%LMN/2)
  call FillWC(CCB%(1),0%,%LMN/2)
  call FillWC(CCP%(1),0%,%LMN/2)
  call FillWC(CCD%(1),0%,%LMN/2)
  call FillWC(CCI%(1),0%,%LMN/2)
  call FillWC(CCG%(1),0%,880%)
'
'
'== Filetypes:   1 = BYT,  2 = DOT,  3 = PCX,  4 = BYR,  5 = CGE
'
'-|-------------<Filetype query loop start>----------------------------------
  do
    print LogOn$(Version$,%Cyan,%Blue)
    print Describe$
    Y% = csrlin
    locate Y%,1
    color %LCyan,%Black
    print Center$(SLText$(21%),80)
    color %Cyan,%Black
    call MLText(2%,6%,"tf")
    Menu1% = Prompt%(50%,Q$)
    select case Q$
      case chr$(27)
	goto LogOff
      case "0"
	Fatal$ = SLText$(22%)
	goto NoJob
      case "1"
        InpExt$ = "BYT"
	exit loop
      case "2"
	InpExt$ = "DOT"
	exit loop
      case "3"
	InpExt$ = "PCX"
	exit loop
      case "4"
	InpExt$ = "BYR"
	exit loop
      case "5"
        InpExt$ = "CGE"
        exit loop
      case else
	print Outch$(Q$)
    end select
  loop
'.|.....................Filetype query loop end..............................
'
'
'-- Get Output path
'
  print LogOn$(Version$,%Brown,%Black);
  print Describe$
  I% = %False
  gosub GetDirPath
  OutPath$ = UsePath$
  if Virgin% then InpPath$ = UsePath$
'
'-|------------------------- Start of Input Path Loop ----------------------
  do
    print LogOn$(Version$,%Green,%Black);
    I% = %True
    gosub GetDirPath			'-- Get input path
    InpPath$ = UsePath$
    A% = ListFiles%(InpPath$,InpExt$,English%)
    if A% then exit loop
    Fatal$ = SLText$(10%)
    if not Redo%(SLText$(11%)) then goto NoJob
  loop
'.|............................... End of Input Path Loop ...................
'
Y% = csrlin
'
'-|---------------------<Filespec gathering Loop Start>----------------------
  do
    locate Y%,1
    InpFile$ = IOFile$(English%,%True,SLText$(49%))
    SelFil$ = InpPath$ + InpFile$
    if FileHere%(SelFil$) then exit loop
    Fatal$ = SLText$(18%) + " " + SelFil$ + " " + SLText$(19%)
    if not Confirm%(SLText$(20%)) then goto NoJob
  loop
'.|....................... Filespec Gathering Loop End ......................
'
'
' Get data for each menu separately
'  1: BYT-Type: CCA% has it bytewise, CCP% has dimensions turned as bytes
'  2: DOT-Type bytes in CCD%. Data size must be 28'000 bytes
'  3: PCX-Type
'  4: BYT-Color assembly Type: BYRed, BYGreen, BYBlue
'  5: CGE-Type: CCB% has it organized, CCG% Image only
'
  DotoTyp$ = Nill$
  RGBFlag% = %False
  select case Menu1%
    case 1						'== BYT
      Z% = GetBytDot%(CCB%(),SelFil$,31680??)
      if Z% then
        if Z% = 1 then
          Fatal$ = SLText$(44%)
        else
          Fatal$ = SLText$(74%)
        end if
        goto NoJob
      end if
      S% = DecBie%(CCA%(1),CCB%(1))
      if S% then
        print SLText$(72%);31680-S%
        delay 1
      end if
      call XDimBA(CCP%(1),CCA%(1),165%,192%)
      call CpyW2W(CCB%(1),CCP%(1),%LMN/2)
      gosub PreView
    case 2						'== DOT
      Z% = GetBytDot%(CCB%(),SelFil$,28000??)
      if Z% then
        if Z% = 1 then
          Fatal$ = SLText$(45%)
        else
          Fatal$ = SLText$(73%)
        end if
        goto NoJob
      end if
      S% = DecBie%(CCD%(1),CCB%(1))
      if S% then
        print SLText$(72%);28000-S%
        delay 1
      end if
      A% = (CCD%(13960) / 256) and 14
      select case A%
	case 8
	  DotoTyp$ = "D"
	case 4
	  DotoTyp$ = "P"
	case 2
	  DotoTyp$ = "I"
      end select
      color %Cyan,%Black
      print SLText$(36%);
      color %LCyan,%Black
      print DotoTyp$
      color %Cyan,%Black
      delay 1.5
      if not ScrMode% = 0 then
	screen ScrMode%
	call StuffV(CCD%(1))
	A$ = KeyIn$
	screen 0
      end if
    case 3						'== PCX
      open SelFil$ for binary as # %UnitAUX
      PLF& = lof(%UnitAUX)
      PCXState% = 1
      get$ # %UnitAUX, 128, RPH$
      call AnaHead(PXH%(),RPH$,StdPal$)
      call DispHead(PXH%(),SelFil$,Version$)
      call ConvtPCX(CCA%(),CCP%(),CCD%(),CCI%(),PXH%(),PLF&)
      if PXH%(3) = 1 then
	call Xpand(CCB%(),CCA%(),CCP%(),CCD%(),CCI%())
	call S1Gray(CCA%(1),StdPal$)
      else
	if PXH%(1) = 5 then
	  if PXH%(10) = 1 then
	    call ExtPalDat(FulPal$,FPF%)
	    if not Confirm%(SLText$(71%)) then gosub RGBF8
	    call E1Gray(CCA%(1),FulPal$)
	  else
	    if not Confirm%(SLText$(71%)) then gosub RGBF24
	    call ColAss(CCB%(1),CCA%(1),CCP%(1),CCD%(1))
	    call CpyW2W(CCA%(1),CCB%(1),%LMN/2)
	  end if
	end if
      end if
      close %UnitAUX
      call CpyW2W(CCP%(1),CCA%(1),%LMN/2)
      call CpyW2W(CCB%(1),CCP%(1),%LMN/2)
      call XDimBA(CCA%(1),CCP%(1),192%,165%)
      gosub PreView
    case 4						'== BYR = BYG = BYB
      RGB$ = StripExt$(SelFil$) + ".BY"
      SelFil$ = RGB$ + "R"
      Z% = GetBytDot%(CCB%(),SelFil$,31680??)
      if Z% then
        if Z% = 1 then
          Fatal$ = SLText$(44%)
        else
          Fatal$ = SLText$(74%)
        end if
        goto NoJob
      end if
      S% = DecBie%(CCD%(1),CCB%(1))
      if S% then
        print SLText$(72%);31680-S%
        delay 1
      end if
      SelFil$ = RGB$ + "G"
      Z% = GetBytDot%(CCB%(),SelFil$,31680??)
      if Z% then
        if Z% = 1 then
          Fatal$ = SLText$(44%)
        else
          Fatal$ = SLText$(74%)
        end if
        goto NoJob
      end if
      S% = DecBie%(CCP%(1),CCB%(1))
      if S% then
        print SLText$(72%);31680-S%
        delay 1
      end if
      SelFil$ = RGB$ + "B"
      Z% = GetBytDot%(CCB%(),SelFil$,31680??)
      if Z% then
        if Z% = 1 then
          Fatal$ = SLText$(44%)
        else
          Fatal$ = SLText$(74%)
        end if
        goto NoJob
      end if
      S% = DecBie%(CCI%(1),CCB%(1))
      if S% then
        print SLText$(72%);31680-S%
        delay 1
      end if
      PCXFil$ = OutPath$ + StripExt$(InpFile$)
      print LogOn$(Version$,%White,%Pink)
      color %LCyan,%Black
      print Center$(SLText$(60%),80)
      color %Cyan,%Black
      call MLText(17%,9%,"tf")
      RGBMenu% = Prompt%(64%,Q$)
      select case RGBMenu%
	case 0, 1					'-- Grayscale BYT
	  call ColAss(CCA%(1),CCD%(1),CCP%(1),CCI%(1))
	  call XDimBA(CCP%(1),CCA%(1),165%,192%)
	  call CpyW2W(CCB%(1),CCP%(1),%LMN/2)
	  RGBFlag% = %False
	  HT% = 3
	  Z$ = PCXFil$ + ".BYC"
	  if RGBMenu% = 1 then
            gosub OptG
	    call XDimBA(CCB%(1),CCA%(1),165%,192%)
          end if
	case 2						'-- Grayscale PCX
	  call Third(CCA%(1),CCD%(1))
	  call Third(CCA%(1),CCP%(1))
	  call Third(CCA%(1),CCI%(1))
	  call XDimBA(CCP%(1),CCA%(1),165%,192%)
	  call CpyW2W(CCB%(1),CCP%(1),%LMN/2)
	  HT% = 3
	  gosub OptIK
	case 3						'-- 8-bit color PCX
	  call Sixth(CCA%(1),CCI%(1),1%)
	  call Sixth(CCA%(1),CCP%(1),6%)
	  call Sixth(CCA%(1),CCD%(1),36%)
          call XDimBA(CCB%(1),CCA%(1),165%,192%)
          call CpyW2W(CCP%(1),CCB%(1),%LMN/2)
	  RGBFlag% = %True
	  HT% = 3
	  gosub OptIK
	case 4						'-- 24-bit PCX
	  call ColAss(CCA%(1),CCD%(1),CCP%(1),CCI%(1))
          Z$ = PCXFil$ + ".PCX"
	  gosub MakeRGBPcx
	  call XDimBA(CCP%(1),CCA%(1),165%,192%)
	  call CpyW2W(CCB%(1),CCP%(1),%LMN/2)
        case 5						'-- 24-Bit PCX
	  call ColAss(CCA%(1),CCD%(1),CCP%(1),CCI%(1))
          Z$ = PCXFil$ + ".TIF"
          gosub MakeRGBTiff
	  call XDimBA(CCP%(1),CCA%(1),165%,192%)
	  call CpyW2W(CCB%(1),CCP%(1),%LMN/2)
        case 6						'-- 6 Planes FITS
          Z$ = PCXFil$ + ".FIT"
          if istrue Existing%(Z$,Fatal$) then
            color %LWhite,%Green
            print " ok "
  	    call ColAss(CCA%(1),CCD%(1),CCP%(1),CCI%(1))
	    call XDimBA(CCB%(1),CCA%(1),165%,192%)
            exit select
          end if
          open Z$ for binary as # %UnitI00
          H% = fileattr(%UnitI00,2)
          print SLText$(59%);" ";Z$
          print " => Head-(dummy)-R-G-B-33/33/33-1/6-30/59/11"
          call FitsHead(FH$())
          print " => Head";
          E% = WrTiff%(CCA%(1),H%,%LMN)			'-- dummy
          A% = E%
          if isfalse E% then print "-(dummy)"; else print "-(?????)";
	  call XDimBA(CCB%(1),CCD%(1),165%,192%)
          E% = WrTiff%(CCB%(1),H%,%LMN)			'-- Red
          A% = A% + E%
          if isfalse E% then print "-R"; else print "-?";
	  call XDimBA(CCB%(1),CCP%(1),165%,192%)
          E% = WrTiff%(CCB%(1),H%,%LMN)			'-- Green
          A% = A% + E%
          if isfalse E% then print "-G"; else print "-?";
	  call XDimBA(CCB%(1),CCI%(1),165%,192%)
          E% = WrTiff%(CCB%(1),H%,%LMN)			'-- Blue
          A% = A% + E%
          if isfalse E% then print "-B"; else print "-?";
	  call Third(CCA%(1),CCD%(1))
	  call Third(CCA%(1),CCP%(1))
	  call Third(CCA%(1),CCI%(1))
	  call XDimBA(CCB%(1),CCA%(1),165%,192%)
          E% = WrTiff%(CCB%(1),H%,%LMN)			'-- 33/33/33
          A% = A% + E%
          if isfalse E% then print "-33/33/33"; else print "-????????";
          call FillWC(CCA%(1),0%,%LMN/2)
	  call Sixth(CCA%(1),CCI%(1),1%)
	  call Sixth(CCA%(1),CCP%(1),6%)
	  call Sixth(CCA%(1),CCD%(1),36%)
          call XDimBA(CCB%(1),CCA%(1),165%,192%)
          E% = WrTiff%(CCB%(1),H%,%LMN)			'-- 3 color 1/6
          A% = A% + E%
          if isfalse E% then print "-1/6"; else print "-???";
	  call ColAss(CCA%(1),CCD%(1),CCP%(1),CCI%(1))
	  call XDimBA(CCB%(1),CCA%(1),165%,192%)
          seek # %UnitI00, 2880				'replace dummy
          E% = WrTiff%(CCB%(1),H%,%LMN)			'-- 30/59/11
          A% = A% + E%
          if isfalse E% then print "-30/59/11"; else print "-????????"
          close %UnitI00
          print
          print SLText$(85%);" ";
          color %LCyan,%Black
          print Z$
          if istrue A% then
            color %LWhite,%Red
            print Center$(str$(A%)+" "+SLText$(86%),79)
            Q$ = KeyIn$
          end if
      end select
      call CpyW2W(CCP%(1),CCB%(1),%LMN/2)
      delay 1
      gosub PreView
    case 5						'== CGE
      Z% = GetCge%(CCB%(),SelFil$)
      if Z% then
        if Z% = 1 then
          Fatal$ = SLText$(76%)
        else
          Fatal$ = SLText$(77%)
        end if
        goto NoJob
      end if
      S% = DecBie%(CCP%(1),CCB%(1))
      if S% then
        print SLText$(72%);1760-S%
        delay 1
      end if
      call CpyW2W(CCA%(1),CCP%(1),880%)		'--for TIFF
      call NiByte(CCB%(1),CCP%(1))
      call CpyW2W(CCG%(1),CCB%(1),1760%)	'--for PCX
      call FillWC(CCP%(1),24672%,%LMN/2)
      call CgeReo(CCP%(1),CCB%(1))
      call CpyW2W(CCB%(1),CCP%(1),%LMN/2)	'--for Preview
      gosub PreView
  end select
  A$ = StripExt$(InpFile$)
  PCLFil$ = OutPath$ + A$
  PCXFil$ = OutPath$ + A$
  TIFFil$ = OutPath$ + A$
  RGBFlag% = %False
  if instat then Q$ = inkey$
'
'
'-|-----------------<Main Menu Loop Start>-----------------------------------
  do
    print LogOn$(Version$,%White,%Blue)
    color %LCyan,%Black
    print Center$(SLText$(27%),80)
    color %Cyan,%Black
    call MLText(3%,0%,"tf")
    A% = Prompt%(39%,Q$)
    select case Q$
      case "A"
	gosub OptA		'Large PCL Image of BYT/PCX-Type file
      case "B"
	gosub OptB		'Small PCL Image of BYT/PCX-Type file
      case "C"
	gosub OptC		'Large PCL Image of DOT-Type file
      case "D"
	gosub OptD		'Small PCL Image of DOT-Type file
      case "E"
        EF% = 19
      	gosub OptEF		'Large PCL Image of CGE-Type file
      case "F"
        EF% = 18
        gosub OptEF		'Small PCL Image of CGE-Type file
      case "G"
	Z$ = PCXFil$ + ".BYP"
	gosub OptG		'PCX --> BYT
      case "H"
        gosub OptH		'BYT/PCX --> 16 PCX
      case "I"
	HT% = 3
	gosub OptIK		'PCX 3.0 Color Image from BYT
      case "J"
	gosub OptJ		'CGE --> 16 PCX
      case "K"
	HT% = 2
	gosub OptIK		'PCX 2.8 Monochrome Image from DOT
      case "L"
        if Confirm%(SLText$(80%)) then L% = 2 else L% = 3
        gosub OptLMNOP		'CGE --> 4-bit TIFF
      case "M","O"
        L% = 1
        gosub OptLMNOP		'BYT and PCX --> 8-bit TIFF
      case "N"
        if Confirm%(SLText$(80%)) then L% = 4 else L% = 5
        gosub OptLMNOP		'CGE --> 4-bit TIFF
      case "P"
        L% = 6
        gosub OptLMNOP		'DOT --> 1-bit TIFF
      case "Q","R","S"
	gosub OptQRS		'BYT/PCX --> CSV - SDF - FXD
      case "T"
	gosub OptT		'BYT/PCX --> SIF
      case "U","V"
	gosub OptUV		'BYT/PCX --> DIF - SDI
      case "W"
	gosub OptW		'BYT/PCX --> CAL
      case "X"
	gosub OptX		'BYT/PCX --> SyLK
      case chr$(13)
	exit loop		'File selection menu
      case chr$(27)
	goto LogOff		'Back to DOS
      case else
	print Outch$(Q$)
    end select
  loop
'.|..................... Main Menu Loop End .................................
'
Virgin% = %False
loop
'
'::::::::::::::::::::::::: End of Main Loop :::::::::::::::::::::::::::::::::
'
' That's it - we're going to log off
'
LogOff:
'
print LogOn$(Version$,%White,%Blue);
print Describe$
color %LWhite,%Green
A$ = VN$ + SLText$(12%)
print Center$(A$,80)
color %LWhite,%Red
A$ = VN$ + "; " + GPCR$
print Center$(A$,80)
color %White,%Black
locate 22,1,1,6,7
goto About
'
'
' Should an unrecoverable error occur, we exit program here
'
NoJob:
'
print
color %Yellow,%Red
A$ = " * " + SLText$(13%) + " - "
print Center$(A$ + Fatal$,80);
color %LWhite,%Red
print Center$(SLText$(14%) + VN$ + SLText$(15%),80);
color %LCyan,%Blue
print Center$(SLText$(16%),80);
color %White,%Black
locate 22,1,1,6,7
A$ = KeyIn$
goto About
'
'
About:
'
print "Info = F1 (2 sec)"
delay 2
if inkey$ = chr$(0,59) then
  print
  color %LWhite,%Black
  print "CCDPRINT.BAS Version";CCDPrintV%
  color %White,%Black
  print "CCDPFUNC.INC Version";CCDPFuncV%
  print "CCDPPROC.INC Version";CCDPProcV%
  print "CCDPSUB .INC Version";CCDPSubV%
  print "CCDPDAT .INC Version";CCDPDatV%
  print "CCDPP   .OBJ Version";CCDPPV%
  print "CCDUP   .OBJ Version";CCDUPV%
  print "IOTEXT  .INC Version";IOTextV%
  print "ERROR3  .INC Version";Error3V%
  print "PB3GPUSE.PBU Version";PB3GPUseV%
  print "ARRAYEXT.PBL Version";ArrayExtV%
  print "CLRSCRN .PBL Version";ClrScrnV%
  print "RLCEGEN .OBJ Version";RlceGenV%
  print "PBSTACK .OBJ Version 100"
  print "PowerBASIC  Revision";PBVer%(SV$);SV$
  print
  color %LWhite,%Blue
  call HoroSoftLogo
  color %White,%Black
  while not instat : wend
  Q$ = inkey$
  call ClsVB
end if
cls
system
'
'===================================-<930212>-=========-<HoroSoft>-==========
