'960210 1516
'****************************************************************************
'* PROGRAM NAME: CCDUFUNC.INC   DATE: 23. 8.1992   INCLUDE IN: CCDUTIL .BAS *
'* LANGUAGE: Power BASIC 3.20   Robert S.Zale                               *
'* PURPOSE: Collection of functions used in CCDUTIL.BAS                     *
'* INCLUDE FILES: none - though they must be included/linked by main        *
'* INPUT FILES: none - files accessed must be opened by main                *
'* OUTPUT FILES: none - files written to must be opened by main             *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.06: 10-FEB-96 Recompiled with PB 3.20                         *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.05:  5-JUN-94 WrtDoto encodes directly from EGA-RAM           *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.04:  1-JUN-94 Adjustments for RLCE-Encoder.                   *
'*                Introduced Version CCDUFuncV% = 204                       *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.03: 30-JAN-94 Added CCD-Gallery Encoded files (CGE)           *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.02: 31-DEC-93 BYT and DOT files are now compressed RLCE       *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.02: 24-DEC-93 Retry on PB3 - added BIE & DOT RLCE             *
'* ------------------------------------------------------------------------ *
'* VERSION: 2.01: 28-MAR-93 Ported to PB 3.00a and back to PB2              *
'* ------------------------------------------------------------------------ *
'* VERSION: 1.00: 23-AUG-92 Copied from CCDPROC.                            *
'* ------------------------------------------------------------------------ *
'* COMMENTS:                                                                *
'* ======================================================================== *
'* AUTHOR: Hans-Ruedi H. Wernli, Pletschgasse, CH-3952 Susten, Switzerland. *
'****************************************************************************
'
'
CCDUFuncV% = 206
'
'
' FUNCTION returns highest possible supported video mode.
'          Supported are VGA (12), MCGA (11), EGA-256K (9) or none (0).
'----------------------------------------------------------------------------
'
function BestVideo%
  local V%,BL%,BH%
  V% = 0
  reg (%AX), &h1A00
  call interrupt %VBIOS
  BL% = reg(%BX) mod 256
  if (BL% = 7) or (BL% = 11) then V% = 11
  if (BL% = 8) or (BL% = 12) then V% = 12
  if V% = 0 then
    reg (%AX), &h1200
    reg (%BX), &h0010
    call interrupt &h10
    BL% = reg(%BX) mod 256
    BH% = fix(reg(%BX) \ 256)
    if BH% = 0 and BL% = 3 then V% = 9 else V% = 0
  end if
  BestVideo% = V%
end function
'
'
' FUNCTION WRite_BYTe writes 8-bit values to BYT-file with RLCE compression
'          Returns zero if ok.
'----------------------------------------------------------------------------
'
function WrtByt%(CCA%(),CCB%(),B$)
  local A%,H%,Z!
  H% = EncBie%(CCB%(1),CCA%(1),31680??)
  Z! = H% * 100 / 31680
  color %Cyan,%Black
  locate csrlin+1,1
  print SLText$(21%);B$;
  open B$ for binary as # %UnitBYT
  H% = fileattr(%UnitBYT,2)
  A% = WrBie(CCB%(1),H%)
  close %UnitBYT
  if not A% then
    print " (";ltrim$(rtrim$(str$(fix(Z!))));"%) "
  else
    print "??";A%
  end if
  color %White,%Black
  WrtByt% = A%
end function
'
'
' FUNCTION writes Dotofile and returns 0 if successfully done
'----------------------------------------------------------------------------
'
function WrtDoto%(CCB%(),A$,Z$)
  local A%,H%,X%,VSize%
  VSize% = 28000
  call VidAry(CCB%(1),X%,VSize%)
  open A$ for binary as # %UnitDOT
  H% = fileattr(%UnitDOT,2)
  A% = WrBie(CCB%(1),H%)
  close %UnitDOT
  Z$ = ""
  WrtDoto% = A%
end function
'
'
' FUNCTION MAXimal_MINimal returns minimum value greater than zero and maximal
'          value of whole array in a formatted string.
'----------------------------------------------------------------------------
'
function MaxMin$(CCA%(),MinV%,MaxV%)
  local A$,B$,C$,D$
  A$ = SLText$(20%)
  MaxV% = MaxByt%(CCA%(1),%LMN)
  MinV% = MinByt%(CCA%(1),%LMN)
  B$ = using$("####",MinV%)
  C$ = using$("####",MaxV%)
  MaxMin$ = A$+B$+" -"+C$+SLText$(169%)+using$("####",MaxV%-MinV%)
end function
'
'
' FUNCTION returns step increment from V%, which is the previous one.
'          If IDF% is true, step is incremented, else decremented.
'----------------------------------------------------------------------------
'
function StepInc%(V%,IDF%)
  local Z%
  Z% = 1
  select case V%
    case 1
      if IDF% then Z% =  2 else Z% =  1
    case 2
      if IDF% then Z% =  5 else Z% =  1
    case 5
      if IDF% then Z% = 10 else Z% =  2
    case 10
      if IDF% then Z% = 20 else Z% =  5
    case 20
      if IDF% then Z% = 50 else Z% = 10
    case 50
      if IDF% then Z% = 50 else Z% = 20
  end select
  StepInc% = Z%
end function
'
'
' FUNCTION returns lower X-position
'---------------------------------------------------------------------------
'
function LowX0%(X0%,A%)
  local Z%
  if A% then
    Z% = X0% - 8
    if Z% > 176 then Z% = 176
  else
    Z% = X0% - 48
    if Z% > 97 then Z% = 97
  end if
  if Z% < 1 then Z% = 1
  LowX0% = Z%
end function
'
'
' FUNCTION returns lower Y-position
'---------------------------------------------------------------------------
'
function LowY0%(Y0%,A%)
  local Z%
  if A% then
    Z% = Y0% - 8
    if Z% > 149 then Z% = 149
  else
    Z% = Y0% - 41
    if Z% > 84 then Z% = 84
  end if
  if Z% <   1 then Z% =   1
  LowY0% = Z%
end function
'
'
' FUNCTION returns new X/Y positions according to Z$
'----------------------------------------------------------------------------
'
function NewXY0%(Z%,S%,Z$)
  local A%
  select case Z$
    case "U"				'UP
      A% = Z% - S%
      if A% < 1 then A% = A% + 165
    case "D"				'DOWN
      A% = Z% + S%
      if A% > 165 then A% = A% - 165
    case "L"				'LEFT
      A% = Z% - S%
      if A% < 1 then A% = A% + 192
    case "R"				'RIGHT
      A% = Z% + S%
      if A% > 192 then A% = A% - 192
  end select
  NewXY0% = A%
end function
'
'
' FUNCTION flashes 3 (%True) or 2 (%False) pictures for Option I (%True) and
'          Option J (%False) and returns key pressed
'----------------------------------------------------------------------------
'
function Flash$(X%,Y%,F%)
  local C1%,C2%,C3%
  if F% then
    C1% = point(X%,Y%)
    C2% = point(X%,Y%+183)
    C3% = point(X%+430,Y%+9)
    while not instat
      pset(X%,Y%),%Black
      pset(X%,Y%+183),%Black
      pset(X%+430,Y%+9),%Black
      delay 0.25
      pset(X%,Y%),%LWhite
      pset(X%,Y%+183),%LWhite
      pset(X%+430,Y%+9),%LWhite
      delay 0.25
    wend
    pset(X%,Y%),C1%
    pset(X%,Y%+183),C2%
    pset(X%+430,Y%+9),C3%
  else
    C1% = point(X%+446,Y%)
    C2% = point(X%+446,Y%+183)
    while not instat
      pset(X%+446,Y%),%Black
      pset(X%+446,Y%+183),%Black
      delay 0.25
      pset(X%+446,Y%),%LWhite
      pset(X%+446,Y%+183),%LWhite
      delay 0.25
    wend
    pset(X%+446,Y%),C1%
    pset(X%+446,Y%+183),C2%
  end if
  Flash$ = inkey$
end function
'
'
' FUNCTION returns byte from CCA% when X and Y are given in option base 1
'----------------------------------------------------------------------------
'
function UseByte%(CCA%(),X%,Y%)
  local A!,A%
  A! = ((X%-1)*165+Y%+1)/2
  A% = fix(A!)
  if A% = A! then
    UseByte% = LoBytI%(CCA%(A%))
  else
    UseByte% = HiBytI%(CCA%(A%))
  end if
end function
'
'
' FUNCTION returns color from ColScale% when word-offset is given
'----------------------------------------------------------------------------
'
function UseColor%(ColScale%(),Z%)
  local A!,A%
  A! = Z% / 2
  A% = fix(A!)
  if A% = A! then
    UseColor% = LoBytI%(ColScale%(A%))
  else
    UseColor% = HiBytI%(ColScale%(A%))
  end if
end function
'
'
' FUNCTION increments file-extension
' ----------------------------------------------------------------------------
'
function IncrExt$(E$)
  local A%,A$,B$,C$
  C$ = left$(E$,2)
  A$ = right$(E$,2)
  A% = val(A$)
  incr A%
  A$ = str$(A%)
  B$ = ltrim$(A$)
  A$ = rtrim$(B$)
  B$ = right$("00"+A$,2)
  IncrExt$ = C$ + B$
end function
'
'
' FUNCTION shows Quadrant assignment. A% has text pointer
'----------------------------------------------------------------------------
'
function Quadrant%(A%)
  local X%,Y%,Q$,Q%
  Y% = csrlin
  X% = pos(Y%)
  locate 19,70
  color %Black,%Cyan
  print "QUADRANT"
  color %Cyan,%Black
  locate 20,70
  print "ͻ"
  locate 21,70
  print " 2  1 "
  locate 22,70
  print "Ķ"
  locate 23,70
  print " 3  4 "
  locate 24,70
  print "ͼ";
  do
    locate Y%,1
    Q% = SelOpt%(SLText$(A%) + " Quadrant (1...4): ")
    if Q% > 0 and Q% < 5 then exit loop
    call Illegal(Q$,SLText$(43%))
  loop
  locate Y%,X%
  Quadrant% = Q%
end function
'
'
' FUNCTION returns free format text for CGE file head
'----------------------------------------------------------------------------
'
function FreeText$
  local A$,H$,T$
  print
  color %Black,%Cyan
  print Center$(SLText$(241%),80)
  color %LCyan,%Black
  print SLText$(240%)
  color %Yellow,%Black
  T$ = " "				'-- blank = free format
  A$ = ""
  do
    line input A$
    T$ = T$ + A$ + chr$(13,10)
    A% = instr(T$,"~")
    if A% then exit loop
  loop
  A$ = left$(T$,A%-1)
  H$ = left$(A$ + space$(255),255)
  print
  color %LWhite,%Cyan
  print H$;
  color %White,%Black
  color %Black,%Cyan
  print "[";chr$(13,10);"]=[CR/LF]";
  color %White,%Black
  call ClrLine
  FreeText$ = H$
end function
'
'
'FUNCTION returns data base formatted text head for CGE files
'----------------------------------------------------------------------------
'
function DbfText$
  local T0$,T1$,T2$,T3$,T4$,T5$,T6$,T7$,T8$,T9$,Q$,Y%
  print
  Y% = csrlin
  call CGETemplate
  T0$ = InsCGETxt$(Y%,11%,%True, %True)		'-- get volume name
  T1$ = InsCGETxt$(Y%,12%,%True, %True)		'-- get file name
  T2$ = InsCGETxt$(Y%,21%,%False,%True)		'-- get subdirs
  T3$ = InsCGETxt$(Y%,46%,%True, %False)	'-- get location
  T4$ = InsCGETxt$(Y%, 8%,%True, %False)	'-- get date
  T5$ = InsCGETxt$(Y%, 5%,%True, %False)	'-- get time
  T6$ = InsCGETxt$(Y%,46%,%True, %False)	'-- get optical system
  T7$ = InsCGETxt$(Y%, 4%,%True, %False)	'-- get focal length
  T8$ = InsCGETxt$(Y%, 5%,%True, %False)	'-- get exposure time
  T9$ = InsCGETxt$(Y%,76%,%True, %False)	'-- get remarks
  DbfText$ = ":" + T0$ + T1$ + T2$ + T3$ + T4$ + T5$ + T6$ + T7$ + T8$ + T9$
end function
'
'
' FUNCTION returns Mac (4 lines) format text for CGE file head if F%=True
'          else 6 lines.
'----------------------------------------------------------------------------
'
function Mac4Text$(F%)
  local Y%,N%,A$,Q$,T$,L1%,L2%,E%
  if istrue F% then
    L1% = 60
    L2% = 62
    T$ = ";"
    E% = 3
  else
    L1% = 42
    L2% = 40
    T$ = "."
    E% = 5
  end if
  Y% = csrlin
  color %Cyan,%Black
  print SLText$(245%);
  locate Y%,13
  print "[";string$(L1%,"_");"]"
  for N% = 1 to E%
    A$ = using$("#",N%)
    print "Text-";A$;" [";string$(L2%,"_");"]"
  next N%
  T$ = T$ + MacLine$(14%,Y%,L1%)
  for N% = 1 to E%
    T$ = T$ + MacLine$(9%,Y%,L2%)
  next N%
  Mac4Text$ = T$
end function
'
'
' FUNCTION prompts and returns for a half formatted string in yellow and
'          echoes it on the screen. Adds CR/LF at end. Size is L%.
'----------------------------------------------------------------------------
'
function MacLine$(X%,Y%,L%)
  color %Yellow,%Black
  locate Y%,X%
  line input Q$
  A$ = left$(Q$+space$(L%),L%)
  locate Y%,X%-1
  call ClrLine
  print A$;
  incr Y%
  MacLine$ = A$ + chr$(13,10)
end function
'
'

' FUNCTION prompts for text in yellow, cuts it to size L% and re-displays it
'          and returns it. FL% is true for left adjust and FU% true
'          for upper case conversion.
'----------------------------------------------------------------------------
'
function InsCGETxt$(Y%,L%,FL%,FU%)
  local A$
  color %Yellow,%Black
  if L% = 76 then
    incr Y%
    locate Y%,4
  else
    locate Y%,24
  end if
  line input Q$
  if FU% then Q$ = ucase$(Q$)
  if FL% then A$ = left$(Q$,L%) else A$ = right$(Q$,L%)
  if not L% = 76 then locate Y%,23 else locate Y%,3
  call ClrLine
  color %LCyan,%Black
  print left$(A$,L%)
  incr Y%
  InsCGETxt$ = left$(A$ + space$(L%),L%) + chr$(13,10)
end function
'
'
' PROCEDURE fills Color, grayscale and convolution arrays
'----------------------------------------------------------------------------
'
sub ArraySetUp(ColScale%(),GrayScale%(),CMasks%(2),CMask$())
  ColScale%( 0) = BytWrd%(%Gray,  %LWhite)	'-- 2-color image, index 0
  ColScale%( 1) = BytWrd%(%Black, %Gray)	'-- 4-color image, index 1
  ColScale%( 2) = BytWrd%(%White, %LWhite)
  ColScale%( 3) = BytWrd%(%Black, %Gray)	'-- 8-color image, index 3
  ColScale%( 4) = BytWrd%(%Cyan,  %White)
  ColScale%( 5) = BytWrd%(%LGreen,%LCyan)
  ColScale%( 6) = BytWrd%(%Yellow,%LWhite)
  ColScale%( 7) = BytWrd%(%Black, %Blue)	'--16-color image, Grayscale
  ColScale%( 8) = BytWrd%(%Red,   %Pink)
  ColScale%( 9) = BytWrd%(%Gray,  %Green)
  ColScale%(10) = BytWrd%(%Brown, %LBlue)
  ColScale%(11) = BytWrd%(%Cyan,  %LRed)
  ColScale%(12) = BytWrd%(%LPink, %White)
  ColScale%(13) = BytWrd%(%LGreen,%LCyan)
  ColScale%(14) = BytWrd%(%Yellow,%LWhite)
  ColScale%(15) = BytWrd%(%Black, %Gray)	'--16-color image, ICI/IBK
  ColScale%(16) = BytWrd%(%Blue,  %LBlue)	'  International Commission
  ColScale%(17) = BytWrd%(%Pink,  %LPink)	'   on Illumination
  ColScale%(18) = BytWrd%(%Red,   %LRed)	'  Internationale
  ColScale%(19) = BytWrd%(%Brown, %Yellow)	'   Beleuchtungskommission
  ColScale%(20) = BytWrd%(%Green, %LGreen)
  ColScale%(21) = BytWrd%(%Cyan,  %LCyan)
  ColScale%(22) = BytWrd%(%White, %LWhite)
  GrayScale%( 0) = NibWrd%(%Black, %Black, %Black, %Black)	'--  0
  GrayScale%( 1) = NibWrd%(%Black, %Gray,  %Black, %Black)	'--  1
  GrayScale%( 2) = NibWrd%(%Gray,  %Gray,  %Black, %Black)	'--  2
  GrayScale%( 3) = NibWrd%(%Gray,  %Gray,  %Black, %Gray)	'--  3
  GrayScale%( 4) = NibWrd%(%Gray,  %Gray,  %Gray,  %Gray)	'--  4
  GrayScale%( 5) = NibWrd%(%Gray,  %White, %Black, %Black)	'--  5
  GrayScale%( 6) = NibWrd%(%Gray,  %White, %Black, %Gray)	'--  6
  GrayScale%( 7) = NibWrd%(%White, %White, %Black, %Gray)	'--  7
  GrayScale%( 8) = NibWrd%(%Black, %LWhite,%Black, %White)	'--  8
  GrayScale%( 9) = NibWrd%(%Gray,  %LWhite,%Black, %White)	'--  9
  GrayScale%(10) = NibWrd%(%White, %White, %Gray,  %White)	'-- 10
  GrayScale%(11) = NibWrd%(%White, %White, %White, %White)	'-- 11
  GrayScale%(12) = NibWrd%(%White, %LWhite,%Gray,  %White)	'-- 12
  GrayScale%(13) = NibWrd%(%LWhite,%LWhite,%Gray,  %White)	'-- 13
  GrayScale%(14) = NibWrd%(%LWhite,%LWhite,%White, %LWhite)	'-- 14
  GrayScale%(15) = NibWrd%(%LWhite,%LWhite,%LWhite,%LWhite)	'-- 15
  CMask$(0) = SLText$(183%)
  CMask$( 1) = "LPF--1"
  CMasks%( 1, 1) =   0			'-- LPF very low power
  CMasks%( 1, 2) =   1
  CMasks%( 1, 3) =   0			'   i1 i2 i3
  CMasks%( 1, 4) =   1			'   i4 i5 i6
  CMasks%( 1, 5) =  16			'   i7 i8 i9
  CMasks%( 1, 6) =   1			'            i10 = sum(i1:i9)
  CMasks%( 1, 7) =   0
  CMasks%( 1, 8) =   1
  CMasks%( 1, 9) =   0
  CMasks%( 1,10) =  20
  CMask$( 2) = "LPF--2"
  CMasks%( 2, 1) =   1				'-- LPF low power
  CMasks%( 2, 2) =   1
  CMasks%( 2, 3) =   1
  CMasks%( 2, 4) =   1
  CMasks%( 2, 5) =  12
  CMasks%( 2, 6) =   1
  CMasks%( 2, 7) =   1
  CMasks%( 2, 8) =   1
  CMasks%( 2, 9) =   1
  CMasks%( 2,10) =  20
  CMask$( 3) = "LPF--3"
  CMasks%( 3, 1) =   1				'-- LPF high power
  CMasks%( 3, 2) =   1
  CMasks%( 3, 3) =   1
  CMasks%( 3, 4) =   1
  CMasks%( 3, 5) =   4
  CMasks%( 3, 6) =   1
  CMasks%( 3, 7) =   1
  CMasks%( 3, 8) =   1
  CMasks%( 3, 9) =   1
  CMasks%( 3,10) =  12
  CMask$( 4) = "LPF-۳-4"
  CMasks%( 4, 1) =   1				'-- LPF very high power
  CMasks%( 4, 2) =   1
  CMasks%( 4, 3) =   1
  CMasks%( 4, 4) =   1
  CMasks%( 4, 5) =   1
  CMasks%( 4, 6) =   1
  CMasks%( 4, 7) =   1
  CMasks%( 4, 8) =   1
  CMasks%( 4, 9) =   1
  CMasks%( 4,10) =   9
  CMask$( 5) = "HPF--1"
  CMasks%( 5, 1) =   0				'-- HPF very low power
  CMasks%( 5, 2) =  -1
  CMasks%( 5, 3) =   0
  CMasks%( 5, 4) =  -1
  CMasks%( 5, 5) =  20
  CMasks%( 5, 6) =  -1
  CMasks%( 5, 7) =   0
  CMasks%( 5, 8) =  -1
  CMasks%( 5, 9) =   0
  CMasks%( 5,10) =  16
  CMask$( 6) = "HPF--2"
  CMasks%( 6, 1) =   0				'-- HPF low power
  CMasks%( 6, 2) =  -1
  CMasks%( 6, 3) =   0
  CMasks%( 6, 4) =  -1
  CMasks%( 6, 5) =  10
  CMasks%( 6, 6) =  -1
  CMasks%( 6, 7) =   0
  CMasks%( 6, 8) =  -1
  CMasks%( 6, 9) =   0
  CMasks%( 6,10) =   6
  CMask$( 7) = "HPF--3"
  CMasks%( 7, 1) =  -1				'-- HPF high power
  CMasks%( 7, 2) =  -1
  CMasks%( 7, 3) =  -1
  CMasks%( 7, 4) =  -1
  CMasks%( 7, 5) =  16
  CMasks%( 7, 6) =  -1
  CMasks%( 7, 7) =  -1
  CMasks%( 7, 8) =  -1
  CMasks%( 7, 9) =  -1
  CMasks%( 7,10) =   8
  CMask$( 8) = "HPF-۳-4"
  CMasks%( 8, 1) =  -1				'-- HPF very high power
  CMasks%( 8, 2) =  -1
  CMasks%( 8, 3) =  -1
  CMasks%( 8, 4) =  -1
  CMasks%( 8, 5) =  10
  CMasks%( 8, 6) =  -1
  CMasks%( 8, 7) =  -1
  CMasks%( 8, 8) =  -1
  CMasks%( 8, 9) =  -1
  CMasks%( 8,10) =   2
  CMask$( 9) = "Laplace-A"
  CMasks%( 9, 1) =   0				'-- Laplace Contour 1
  CMasks%( 9, 2) =  -1
  CMasks%( 9, 3) =   0
  CMasks%( 9, 4) =  -1
  CMasks%( 9, 5) =   4
  CMasks%( 9, 6) =  -1
  CMasks%( 9, 7) =   0
  CMasks%( 9, 8) =  -1
  CMasks%( 9, 9) =   0
  CMasks%( 9,10) =   0
  CMask$(10) = "Laplace-B"
  CMasks%(10, 1) =   1				'-- Laplace Contour 2
  CMasks%(10, 2) =  -2
  CMasks%(10, 3) =   1
  CMasks%(10, 4) =  -2
  CMasks%(10, 5) =   4
  CMasks%(10, 6) =  -2
  CMasks%(10, 7) =   1
  CMasks%(10, 8) =  -2
  CMasks%(10, 9) =   1
  CMasks%(10,10) =   0
  CMask$(11) = "Sobel-"
  CMasks%(11, 1) =   1				'-- Sobel Contour Left
  CMasks%(11, 2) =   0
  CMasks%(11, 3) =  -1
  CMasks%(11, 4) =   2
  CMasks%(11, 5) =   0
  CMasks%(11, 6) =  -2
  CMasks%(11, 7) =   1
  CMasks%(11, 8) =   0
  CMasks%(11, 9) =  -1
  CMasks%(11,10) =   0
  CMask$(12) = "Sobel-"
  CMasks%(12, 1) =  -1				'-- Sobel Contour Top
  CMasks%(12, 2) =  -2
  CMasks%(12, 3) =  -1
  CMasks%(12, 4) =   0
  CMasks%(12, 5) =   0
  CMasks%(12, 6) =   0
  CMasks%(12, 7) =   1
  CMasks%(12, 8) =   2
  CMasks%(12, 9) =   1
  CMasks%(12,10) =   0
  CMask$(13) = "Sobel-"
  CMasks%(13, 1) =  -2				'-- Sobel Top Left
  CMasks%(13, 2) =  -1
  CMasks%(13, 3) =   0
  CMasks%(13, 4) =  -1
  CMasks%(13, 5) =   0
  CMasks%(13, 6) =   1
  CMasks%(13, 7) =   0
  CMasks%(13, 8) =   1
  CMasks%(13, 9) =   2
  CMasks%(13,10) =   0
  CMask$(14) = "Prewitt-"
  CMasks%(14, 1) =   1				'-- Prewitt Left
  CMasks%(14, 2) =   0
  CMasks%(14, 3) =  -1
  CMasks%(14, 4) =   1
  CMasks%(14, 5) =   0
  CMasks%(14, 6) =  -1
  CMasks%(14, 7) =   1
  CMasks%(14, 8) =   0
  CMasks%(14, 9) =  -1
  CMasks%(14,10) =   0
  CMask$(15) = "Prewitt-"
  CMasks%(15, 1) =  -1				'-- Prewitt Top
  CMasks%(15, 2) =  -1
  CMasks%(15, 3) =  -1
  CMasks%(15, 4) =   0
  CMasks%(15, 5) =   0
  CMasks%(15, 6) =   0
  CMasks%(15, 7) =   1
  CMasks%(15, 8) =   1
  CMasks%(15, 9) =   1
  CMasks%(15,10) =   0
end sub
'
'
'=============================-<920823>-=============-<HoroSoft>-=============
'
